/*
Pps -- Plot for PostScript printer

Library for PostScript output
current version: PostScript Level2, version 3.0, Encapsulated PS

1997/10/26 T.Handa ver.1.00
1997/12/04 T.Handa ver.1.03 (debug for NewsC)
1999/10/02 T.Handa ver.1.12 (debug) for Linux, tempolary PS file name allocation
2000/ 1/12 T.Handa ver.1.20 (revise) PpsInitialize(); landscape parameter input
2000/ 2/15 T.Handa ver.1.30 (revise) PpsDoPrint(); no print for a permanent PS file
2000/ 2/15 T.Handa ver.1.31 (debug) PpsDrawString(), PpsDrawJustifiedString(); string with "(" and ")"
2000/ 2/15 T.Handa ver.1.50 (modify) PpsPrintPage(),PpsInitialize(); PS3.0,
 (add) PpsSetWholePageSize();
*/
#define PPS_VERSION "1.50"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <time.h>

#include <Ppslib.h>
 
#ifdef _NO_PROTO
extern char *mktemp();
#else
extern char *mktemp(char *);
#endif

typedef	struct {
  double x;
  double y;} PPS_XY_POS;
typedef struct {
  double list[N_PATT_LIST];
  int n;
  double phase;} LINE_PATTERN;


static PPS_XY_POS PpsView_WholePageBLC={0.0,0.0};
static PPS_XY_POS PpsView_WholePageTRC={576.0,288.0};

static FILE	*PpsView_fp;
static double	PpsView_linewidth=1.0, PpsView_density=1.0;
static PPS_XY_POS	PpsView_origin={0.0,0.0};
static PPS_XY_POS	PpsView_scale={1.0,1.0};
static double	PpsView_rgb[3]={0.0,0.0,0.0};
static short	PpsView_fontsize=14;

static char	*tmp_filename;

static char	PpsView_printer[256];
static LINE_PATTERN	PpsView_line_patt;

#define	PpsCm2PointX(xx)	((xx)+PpsView_origin.x)*CM2INCH*DVI_SCALE*PpsView_scale.x
#define	PpsCm2PointY(yy)	((yy)+PpsView_origin.y)*CM2INCH*DVI_SCALE*PpsView_scale.y
#define	PpsGetPointSpanX(xx)	(xx)*CM2INCH*DVI_SCALE*PpsView_scale.x
#define	PpsGetPointSpanY(yy)	(yy)*CM2INCH*DVI_SCALE*PpsView_scale.y


static void	PpsNewPath()
/*
Initialize pen path
1997/10/13 T.Handa ver.1.00
*/
{
  int ii;

  fprintf(PpsView_fp,"newpath\n");

  if(PpsView_rgb[0]>0.0 || PpsView_rgb[1]>0.0 || PpsView_rgb[2]>0.0) {
    fprintf(PpsView_fp,"%f %f %f setrgbcolor\n",PpsView_rgb[0],PpsView_rgb[1],PpsView_rgb[2]);
  }

  return;
}

/* administration functions: get global parameters */
void PpsGetScale(x,y)
double *x,*y;
/*
get the view scale
1997/10/21 T.Handa ver.1.00
*/
{
  *x=PpsView_scale.x;
  *y=PpsView_scale.y;

  return;
}
void PpsGetOrigin(x,y)
double *x,*y;
/*
get the view origin
1997/10/20 T.Handa ver.1.00
*/
{
  *x=PpsView_origin.x;
  *y=PpsView_origin.y;

  return;
}

/* administration functions: put global parameters */
void PpsSetPrinterName(prt_name)
char prt_name[];
/*
output printer name
1997/10/13 T.Handa ver.1.00
*/
{
  strcpy(PpsView_printer,prt_name);

  return;
}

void	PpsPutScale(x,y)
double x,y;
/*
set view scale
1997/10/21 T.Handa ver.1.00
*/
{
  PpsView_scale.x=x;
  PpsView_scale.y=y;

  return;
}
void	PpsPutOrigin(x,y)
double x,y;
/*
set view origin
1997/10/21 T.Handa ver.1.00
*/
{
  PpsView_origin.x=x;
  PpsView_origin.y=y;

  return;
}

void PpsSetWholePageSize(x1,y1,x2,y2)
/*
set page size
2000/ 2/15 T.Handa ver.1.00
*/
double x1,y1,x2,y2;
{
  PpsView_WholePageBLC.x=x1;
  PpsView_WholePageBLC.y=y1;
  PpsView_WholePageTRC.x=x2;
  PpsView_WholePageBLC.y=y2;
  return;
}


/* draw graphics */

void PpsSetColor(color)
char color[];
/*
set line color
1997/10/12 T.Handa ver.1.00
*/
{
	

  return;
}

void PpsSetGray(density)
double	density;
/*
set line gray
1997/10/15 T.Handa ver.1.00
*/
{
  PpsView_density=1.0-density;
  fprintf(PpsView_fp,"%f setgray\n",1.0-density);

  return;
}

void PpsSetLineWidth(linewidth)
double linewidth;
/*
set line width
1997/10/15 T.Handa ver.1.00
*/
{
  PpsView_linewidth=linewidth;
  fprintf(PpsView_fp,"%f setlinewidth\n",linewidth);

  return;
}

void PpsSetLinePattern(line_patt,n_patt,phase)
double line_patt[];
int n_patt;
double phase;
/*
set line patern
1997/10/16 T.Handa ver.1.00
*/
{
  int ii;

  if(n_patt<2) { /* solid line */
    fprintf(PpsView_fp,"[ ] 0 setdash\n");
    return;
  }

  if(n_patt>N_PATT_LIST) n_patt=N_PATT_LIST;
  for(ii=0;ii<n_patt;ii++)
    PpsView_line_patt.list[ii]=PpsGetPointSpanX(line_patt[ii]);
  PpsView_line_patt.n=n_patt;
  PpsView_line_patt.phase=PpsGetPointSpanX(phase);	

  fprintf(PpsView_fp,"[ ");
  for(ii=0;ii<PpsView_line_patt.n;ii++)
    fprintf(PpsView_fp,"%f ",PpsView_line_patt.list[ii]);
  fprintf(PpsView_fp,"] %f ",PpsView_line_patt.phase);
  fprintf(PpsView_fp,"setdash\n");

  return;
}
void PpsDrawLine(x1,y1,x2,y2)
double x1,y1,x2,y2;
/*
draw a straight line
1997/10/12 T.Handa ver.1.00
*/
{
  double xx1,xx2,yy1,yy2;

  xx1=PpsCm2PointX(x1);
  xx2=PpsCm2PointX(x2);
  yy1=PpsCm2PointY(y1);
  yy2=PpsCm2PointY(y2);

  PpsNewPath();
  fprintf(PpsView_fp,"%f %f moveto\n",xx1,yy1);
  fprintf(PpsView_fp,"%f %f lineto\n",xx2,yy2);
  fprintf(PpsView_fp,"stroke\n\n");

  return;
}
void PpsDrawLines(pos_list,n_pt)
PpsPoint pos_list[];
int	n_pt;
/*
draw line segments
1997/10/23 T.Handa ver.1.00
*/
{
  double xx,yy;
  int	ii;

  xx=PpsCm2PointX(pos_list[0].x);
  yy=PpsCm2PointY(pos_list[0].y);

  PpsNewPath();
  fprintf(PpsView_fp,"%f %f moveto\n",xx,yy);

  for(ii=1;ii<n_pt;ii++) {
    xx=PpsCm2PointX(pos_list[ii].x);
    yy=PpsCm2PointY(pos_list[ii].y);
    fprintf(PpsView_fp,"%f %f lineto\n",xx,yy);
  }

  fprintf(PpsView_fp,"stroke\n\n");

  return;
}

void PpsDrawRectangle(x1,y1,x2,y2,fill)
double x1,y1,x2,y2;
int	fill;
/*
draw a rectangle
1997/10/12 T.Handa ver.1.00
*/
{
  double xx1,xx2,yy1,yy2;

  xx1=PpsCm2PointX(x1);
  xx2=PpsCm2PointX(x2);
  yy1=PpsCm2PointY(y1);
  yy2=PpsCm2PointY(y2);

  PpsNewPath();
  fprintf(PpsView_fp,"%f %f moveto\n",xx1,yy1);
  fprintf(PpsView_fp,"%f %f lineto\n",xx2,yy1);
  fprintf(PpsView_fp,"%f %f lineto\n",xx2,yy2);
  fprintf(PpsView_fp,"%f %f lineto\n",xx1,yy2);
  fprintf(PpsView_fp,"closepath\n");
  if(fill==NULL)
    fprintf(PpsView_fp,"stroke\n\n");
  else
    fprintf(PpsView_fp,"fill\n\n");

  return;
}
void PpsDrawPolygon(pos_list,n_pt,fill)
PpsPoint pos_list[];
int	n_pt;
int	fill;
/*
draw a polygon
1997/10/19 T.Handa ver.1.00
*/
{
  double xx,yy;
  int	ii;

  xx=PpsCm2PointX(pos_list[0].x);
  yy=PpsCm2PointY(pos_list[0].y);

  PpsNewPath();
  fprintf(PpsView_fp,"%f %f moveto\n",xx,yy);

  for(ii=1;ii<n_pt;ii++) {
    xx=PpsCm2PointX(pos_list[ii].x);
    yy=PpsCm2PointY(pos_list[ii].y);
    fprintf(PpsView_fp,"%f %f lineto\n",xx,yy);
  }

  fprintf(PpsView_fp,"closepath\n");
  if(fill==NULL) {
    fprintf(PpsView_fp,"stroke\n\n");
  } else {
    fprintf(PpsView_fp,"fill\n\n");
  }

  return;
}

void PpsDrawArc(x1,y1,radius,start,theta,fill)
double x1,y1,radius;
double start,theta;
int fill;
/*
draw an arc
1997/10/13 T.Handa ver.1.00
*/
{
  double xx1,xx2,yy1,yy2;
  double rr;

  xx1=PpsCm2PointX(x1);
  yy1=PpsCm2PointY(y1);
  rr=PpsGetPointSpanX(radius);

  PpsNewPath();

  fprintf(PpsView_fp,"%f %f %f %f %f ",xx1,yy1,rr,start,start+theta);

  if(fill!=NULL) fprintf(PpsView_fp,"closepath\n");

  if(theta>0.0)
    fprintf(PpsView_fp,"arc\n");
  else
    fprintf(PpsView_fp,"arcn\n");

  if(fill==NULL)
    fprintf(PpsView_fp,"stroke\n\n");
  else
    fprintf(PpsView_fp,"fill\n");

  return;
}

void PpsDrawSector(x1,y1,radius,start,theta,fill)
double x1,y1,radius;
double start,theta;
int	fill;
/*
draw a sector
1997/10/13 T.Handa ver.1.00
*/
{
  double xx1,xx2,yy1,yy2;
  double rr;

  xx1=PpsCm2PointX(x1);
  yy1=PpsCm2PointY(y1);
  rr=PpsGetPointSpanX(radius);

  PpsNewPath();
  fprintf(PpsView_fp,"%f %f moveto\n",xx1,yy1);
  fprintf(PpsView_fp,"%f %f %f %f %f ",xx1,yy1,rr,start,start+theta);
  if(theta>0.0)
    fprintf(PpsView_fp,"arc\n");
  else
    fprintf(PpsView_fp,"arcn\n");

  fprintf(PpsView_fp,"closepath\n");
  if(fill==NULL)
    fprintf(PpsView_fp,"stroke\n\n");
  else
    fprintf(PpsView_fp,"fill\n\n");

  return;
}

/* plot a character */

void PpsSetFont(font_name,size)
char font_name[];
short size;
/*
set font and its size
1997/10/12 T.Handa ver.1.00
*/
{
  fprintf(PpsView_fp,"/%s findfont\n",font_name);
  fprintf(PpsView_fp,"%d scalefont\n",size);
  fprintf(PpsView_fp,"setfont\n");

  PpsView_fontsize=size;

  return;
}

void PpsDrawString(x1,y1,draw_str)
double x1,y1;
char draw_str[];
/*
draw characters
1997/10/12 T.Handa ver.1.00
*/
{
  double xx,yy;
  char *buffer;
  int ii,jj,l_str,n_paren;

  xx=PpsCm2PointX(x1);
  yy=PpsCm2PointY(y1);
  fprintf(PpsView_fp,"%f %f moveto\n",xx,yy);

  if(PpsView_rgb[0]!=0.0 || PpsView_rgb[1]!=0.0 || PpsView_rgb[2]!=0.0) {
    fprintf(PpsView_fp,"%f %f %f setrgbcolor\n",PpsView_rgb[0],PpsView_rgb[1],PpsView_rgb[2]);
  }

  l_str=strlen(draw_str);
  n_paren=0;
  for(ii=0;ii<l_str;ii++) {
    if(draw_str[ii]=='(' || draw_str[ii]==')')
      n_paren++;
  }
  if(n_paren==0)
    fprintf(PpsView_fp,"(%s) show\n",draw_str);
  else {
    buffer=malloc(l_str+n_paren+1);
    for(ii=0,jj=0;ii<l_str;ii++,jj++) {
      if(draw_str[ii]=='(' || draw_str[ii]==')') {
	buffer[jj]='\\'; jj++;
	buffer[jj]=draw_str[ii];
      } else
	buffer[jj]=draw_str[ii];
    }
    buffer[l_str+n_paren]=NULL;
    fprintf(PpsView_fp,"(%s) show\n",buffer);
    free(buffer);
  }
  fprintf(PpsView_fp,"\n");

  return;
}


void PpsDrawJustifiedString(x1,y1,draw_str,x_style,y_style)
double x1,y1;
char draw_str[];
char x_style[],y_style[];
/*
draw strings with justifying their position
1997/10/22 T.Handa ver.1.00
*/
{
  double xx,yy;
  char *buffer;
  int ii,jj,l_str,n_paren;

  xx=PpsCm2PointX(x1);
  yy=PpsCm2PointY(y1);
  fprintf(PpsView_fp,"%f %f moveto\n",xx,yy);

/* shift yy because PostScript command cannot accept top justfy.
   assume net height of the font is 71% of its size */
  if(strcmp(y_style,"top")==0) {
    yy=(double)PpsView_fontsize;
    yy= -yy*0.71;
    fprintf(PpsView_fp,"0 %f rmoveto \n",yy);
  } else if(strcmp(y_style,"center")==0) {
    yy=(double)PpsView_fontsize;
    yy= -yy*0.71;
    fprintf(PpsView_fp,"0 %f rmoveto \n",yy*0.5);
  } else if(strcmp(y_style,"hanging")==0 || strcmp(y_style,"hang")==0) {
    yy=(double)PpsView_fontsize;
    yy= -yy;
    fprintf(PpsView_fp,"0 %f rmoveto \n",yy);
  }


  if(PpsView_rgb[0]!=0.0 || PpsView_rgb[1]!=0.0 || PpsView_rgb[2]!=0.0) {
    fprintf(PpsView_fp,"%f %f %f setrgbcolor\n",
	    PpsView_rgb[0],PpsView_rgb[1],PpsView_rgb[2]);
  }

  l_str=strlen(draw_str);
  n_paren=0;
  for(ii=0;ii<l_str;ii++) {
    if(draw_str[ii]=='(' || draw_str[ii]==')') n_paren++;
  }
  if(n_paren==0)
    fprintf(PpsView_fp,"(%s) ",draw_str);
  else {
    buffer=malloc(l_str+n_paren+1);
    for(ii=0,jj=0;ii<l_str;ii++,jj++) {
      if(draw_str[ii]=='(' || draw_str[ii]==')') {
	buffer[jj]='\\'; jj++;
	buffer[jj]=draw_str[ii];
      } else
	buffer[jj]=draw_str[ii];
    }
    buffer[l_str+n_paren]=NULL;
    fprintf(PpsView_fp,"(%s) ",buffer);
    free(buffer);
  }

/* only R/L justify because PostScript command cannot accept top and bottom justify. */
  if(strcmp(x_style,"right")==0)
    fprintf(PpsView_fp,"dup stringwidth pop 0 exch sub 0 rmoveto ");
  else if(strcmp(x_style,"center")==0)
    fprintf(PpsView_fp,"dup stringwidth pop 2 div 0 exch sub 0 rmoveto ");

/* justify using PostScript command */
/*
  if(strcmp(x_style,"right")==0 || strcmp(x_style,"center")==0 || strcmp(y_style,"top")==0 || strcmp(y_style,"center")==0) {
    fprintf(PpsView_fp,"dup stringwidth ");
    if(strcmp(y_style,"center")==0)
      fprintf(PpsView_fp,"2 div 0 exch sub ");
    else if(strcmp(y_style,"top")==0)
      fprintf(PpsView_fp,"0 exch sub ");
    else	
      fprintf(PpsView_fp,"pop 0 ");

    fprintf(PpsView_fp,"exch ");

    if(strcmp(x_style,"center")==0)
      fprintf(PpsView_fp,"2 div 0 exch sub ");
    else if(strcmp(x_style,"right")==0)
      fprintf(PpsView_fp,"0 exch sub ");
    else
      fprintf(PpsView_fp,"pop 0 ");
    
    fprintf(PpsView_fp,"exch rmoveto ");
  }
*/

  fprintf(PpsView_fp,"show \n\n");

  return;
}


/* set end of the page */

void PpsPrintPage()
/*
1997/10/26 T.Handa ver.1.00
2000/ 2/15 T.Handa ver.1.50 (modify) PS level2 ver.3.0 EPSF
*/
{
  fprintf(PpsView_fp,"grestore\n");
  fprintf(PpsView_fp,"showpage\n");

  fprintf(PpsView_fp,"%%%%End of Pxp/Pps PostScript output\n");
  fprintf(PpsView_fp,"%%%%Trailer\n");

  return;
}

/* initialize PostScript output */
static void read_setup_file(whole_blc,whole_trc)
PPS_XY_POS *whole_blc,*whole_trc;
{
  FILE *fp;
  static char filename[256];
  char buffer[256],*ret_char,*token;

  fp=NULL;
  ret_char=getenv("PPS_SETUP_FILE");
  if(ret_char!=NULL) {
    strcpy(filename,ret_char);
  } else {
    ret_char=getenv("HOME");
    if(ret_char!=NULL) {
      sprintf(filename,"%s/.pps_setup",ret_char);
      fp=fopen(filename,"r");
      if(fp==NULL) ret_char==NULL;
    }
    if(ret_char==NULL) {
      strcpy(filename,"/usr/local/Pxplib/lib/.pps_setup");
    }
  }
  if(fp==NULL)
    fp=fopen(filename,"r");
  if(fp!=NULL) {
    ret_char=fgets(buffer,256,fp);
    if(ret_char!=0) {
      token=strtok(buffer,", \t\n\r/");
      if(token!=NULL) {
	PpsView_WholePageBLC.x=atoi(token);
	token=strtok(NULL,", \t\n\r/");
	if(token!=NULL) PpsView_WholePageBLC.y=atoi(token);
      }
    }
    ret_char=fgets(buffer,256,fp);
    if(ret_char!=0) {
      token=strtok(buffer,", \t\n\r/");
      if(token!=NULL) {
	PpsView_WholePageTRC.x=atoi(token);
	token=strtok(NULL,", \t\n\r/");
	if(token!=NULL) PpsView_WholePageTRC.y=atoi(token);
      }
    }
    fclose(fp);
  }
  return;
}


void PpsInitialize(output_file,width,height)
/*
1999/10/2 T.Handa (debug) for Linux, tempolary PS file name allocation
2000/01/12 T.Handa ver.1.20 (revise) read the landscape parameter from the file
2000/ 2/15 T.Handa ver.1.50 (modify) PS level2 ver.3.0 EPSF
*/
char *output_file;
double width,height;
{
/*  double	width_in_dot,height_in_dot; */
  long	utc;
  static char tmpPSfilename[256];
  static int hash_flag=0;
  FILE *fp;

  if(hash_flag==0) {
    read_setup_file(&PpsView_WholePageBLC,&PpsView_WholePageTRC);
    hash_flag=1;
  }

  if(output_file==NULL) {
    strcpy(tmpPSfilename,"/tmp/PostScriptXXXXXX");
    tmp_filename=mktemp(tmpPSfilename);
    PpsView_fp=fopen(tmp_filename,"w");
  } else if(output_file[0]==NULL || strcmp(output_file,"-")==0) {
    tmp_filename=output_file;
    PpsView_fp=stdout;
  } else {
    tmp_filename=output_file;
    PpsView_fp=fopen(output_file,"w");
  }

  time(&utc);

/*  fprintf(PpsView_fp,"%%!PS-Adobe-2.0\n"); */
  fprintf(PpsView_fp,"%%!PS-Adobe-3.0 EPSF-3.0\n");
  fprintf(PpsView_fp,"%%%%BoundingBox:%f %f %f %f\n",PpsView_WholePageBLC.x,PpsView_WholePageBLC.y,PpsView_WholePageTRC.x,PpsView_WholePageTRC.y);
  fprintf(PpsView_fp,"%%%%LanguageLevel: 2\n");
  fprintf(PpsView_fp,"%%%%Creator:Pxp/Pps library ver.%s\n",PPS_VERSION);
  fprintf(PpsView_fp,"%%%%Pxp/Pps is programmed by Toshihiro Handa.\n");
  fprintf(PpsView_fp,"%%%%CreationDate:%s",ctime(&utc));
  fprintf(PpsView_fp,"%%%%EndComments\n");
  fprintf(PpsView_fp,"%%%%Pages:1\n");
  fprintf(PpsView_fp,"%%%%EndProlog\n");
  fprintf(PpsView_fp,"%%%%Page 1 1\n");
  fprintf(PpsView_fp,"%%%%\n%%%%\n\n");

  fprintf(PpsView_fp,"gsave\n");

  fprintf(PpsView_fp,"/Courier findfont\n");
  fprintf(PpsView_fp,"14 scalefont\n");
  fprintf(PpsView_fp,"setfont\n");
  fprintf(PpsView_fp,"0.0 0.0 moveto\n");
  fprintf(PpsView_fp,"( ) show\n\n");

  if(PpsView_scale.x!=1.0 || PpsView_scale.y!=1.0)
    fprintf(PpsView_fp,"%f %f scale\n\n",PpsView_scale.x,PpsView_scale.y);

  if(width>height) {	/* landscape */
    fprintf(PpsView_fp,"%d %d translate\n",(long)PpsView_WholePageTRC.x,(long)PpsView_WholePageBLC.y);
    fprintf(PpsView_fp,"90 rotate\n\n");
  }

  return;
}

void PpsDoPrint()
/*
execute print command with PostScript printer
1997/10/14 T.Handa ver.1.00
2000/ 2/15 T.Handa ver.1.30 (revise) no print for a permanent PS file
*/
{
  char	*prt_flg;
  int	l_prtname,l_fname,l_cmdstr;

  char	*lpr_cmd;

  if(PpsView_fp==stdout) return;

  fclose(PpsView_fp);	/* close PS data file */

  if(strncmp(tmp_filename,"/tmp/PostScript",15)!=0)
    return;

  l_prtname=strlen(PpsView_printer);
  l_fname=strlen(tmp_filename);
  l_cmdstr=4+2+l_prtname+1+l_fname+1+1+3+l_fname;
  lpr_cmd=(char *)malloc(sizeof(char)*l_cmdstr);
  if(l_prtname==0)
    sprintf(lpr_cmd,"lpr %s;rm %s",tmp_filename,tmp_filename);
  else
    sprintf(lpr_cmd,"lpr -P%s %s;rm %s",PpsView_printer,tmp_filename,tmp_filename);
  system(lpr_cmd);
  free(lpr_cmd);

  return;
}
