/*
PxpG -- Graph Plot for X11 window and/or Postscript printer

Library for X11 display and/or PostScript printer
1997/10/26 T.Handa ver.1.00
1998/ 5/18 T.Handa ver.1.10 (add PxpGFilledBox();)
1999/10/ 2 F.Yamamoto ver.1.12 (debug) PxpGTickMark(),PxpGTickMarkInset(),PxpGLabel(),PxpGLabelInset();
2000/ 2/14 T.Handa ver.1.22 (revise) shrink PxpGGrid(),PxpGTickMark(),PxpGLabel();
2000/ 2/15 T.Handa ver.1.51 (modify) PxpGGridInset(),PxpGTickMarkInset(),PxpGLabelInset(); 
                            (debug) PxpGCeilingRange();
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <math.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include <Xm/DrawingA.h>
#include <Xm/MainW.h>
#include <Xm/RowColumn.h>
#include <Xm/CascadeB.h>
#include <Xm/PushBG.h>

#include <Pxplib.h>
#include <PxpGlib.h>

#define	BUFFER_SIZE	256

static PxpPoint PxpGView_scale={1.0,1.0};
static PxpPoint PxpGView_origin={0.0,0.0};
static PxpPoint PxpGView_physics_origin={0.0,0.0};
static int PxpGView_prjection_type=CARTESIAN;
static double PxpGView_plot_step=0.1;

/* To estimate position correspondence */
void PxpGGetScale(x,y)
/*
get a graph scale (plot unit/physics unit)
1997/10/21 T.Handa ver.1.00
*/
double *x, *y;
{
  *x=PxpGView_scale.x;
  *y=PxpGView_scale.y;

  return;
}
void PxpGGetPlotOrigin(x,y)
double *x, *y;
/*
get a graph origin (plot unit)
1997/10/21 T.Handa
*/
{
  *x=PxpGView_origin.x;
  *y=PxpGView_origin.y;

  return;
}
void PxpGGetPhysicsOrigin(x,y)
double *x, *y;
/*
get a graph origin (physics unit)
1997/10/21 T.Handa ver.1.00
*/
{
  *x=PxpGView_physics_origin.x;
  *y=PxpGView_physics_origin.y;

  return;
}

void PxpGPutScale(x,y)
double x, y;
/*
set a graph scale (plot unit/physics unit)
1997/10/21 T.Handa ver.1.00
*/
{
  PxpGView_scale.x=x;
  PxpGView_scale.y=y;

  return;
}
void PxpGPutPlotOrigin(x,y)
double x, y;
/*
set a graph origin (plot unit)
1997/10/21 T.Handa ver.1.00
*/
{
  PxpGView_origin.x=x;
  PxpGView_origin.y=y;

  return;
}
void PxpGPutPhysicsOrigin(x,y)
double x, y;
/*
set a graph origin (physics unit)
1997/10/21 T.Handa ver.1.00
*/
{
  PxpGView_physics_origin.x=x;
  PxpGView_physics_origin.y=y;

  return;
}

void PxpGSetRange(phys0,phys1,cm0,cm1)
PxpPoint phys0,phys1,cm0,cm1;
/*
set a scale and origin by corresponding two points
1997/10/21 T.Handa ver.1.00
*/
{
  PxpGView_origin.x=cm0.x;
  PxpGView_origin.y=cm0.y;
  PxpGView_physics_origin.x=phys0.x;
  PxpGView_physics_origin.y=phys0.y;

  PxpGView_scale.x=(cm1.x-cm0.x)/(phys1.x-phys0.x);
  PxpGView_scale.y=(cm1.y-cm0.y)/(phys1.y-phys0.y);

  return;
}

PxpPoint PxpGPhysicsValue(cm)
PxpPoint cm;
/*
get a physics value at a graph position (plot unit)
1997/10/21 T.Handa ver.1.00
*/
{
  PxpPoint phys;

  phys.x=(cm.x-PxpGView_origin.x)/PxpGView_scale.x+PxpGView_physics_origin.x;
  phys.y=(cm.y-PxpGView_origin.y)/PxpGView_scale.y+PxpGView_physics_origin.y;

  return(phys);
}
PxpPoint PxpGPlotPosition(phys)
PxpPoint phys;
/*
get a graph position with a physics value
1997/10/21 T.Handa ver.1.00
*/
{
  PxpPoint cm;

  cm.x=(phys.x-PxpGView_physics_origin.x)*PxpGView_scale.x+PxpGView_origin.x;
  cm.y=(phys.y-PxpGView_physics_origin.y)*PxpGView_scale.y+PxpGView_origin.y;

  return(cm);
}
PxpPoint PxpGPhysicsSpan(cm)
PxpPoint cm;
/*
get a physics value difference for a graph length
1997/10/21 T.Handa ver.1.00
*/
{
  PxpPoint phys;

  phys.x=cm.x/PxpGView_scale.x;
  phys.y=cm.y/PxpGView_scale.y;

  return(phys);
}
PxpPoint PxpGPlotSpan(phys)
PxpPoint phys;
/*
get a graph length for a physics value difference
1997/10/21 T.Handa ver.1.00
*/
{
  PxpPoint cm;

  cm.x=phys.x*PxpGView_scale.x;
  cm.y=phys.y*PxpGView_scale.y;

  return(cm);
}

/* To plot a graph */

void PxpGBox(phys_min,phys_max)
PxpPoint phys_min,phys_max; /* <in> left and right position (phys unit); min=left,bottom; max=right,top */
/*
plot a simple lectangle frame in physics unit
1997/10/21 T.Handa ver.1.00
*/
{
  PxpPoint cm_min,cm_max;

  cm_min=PxpGPlotPosition(phys_min);
  cm_max=PxpGPlotPosition(phys_max);

  PxpDrawRectangle(cm_min.x,cm_min.y,cm_max.x,cm_max.y,0);	

  return;
}

void PxpGFilledBox(phys_min,phys_max)
PxpPoint phys_min,phys_max; /* <in> left and right position (phys unit); min=left,bottom; max=right,top */
/*
plot a simple filled box in physics unit
1997/10/21 T.Handa ver.1.00
*/
{
  PxpPoint cm_min,cm_max;

  cm_min=PxpGPlotPosition(phys_min);
  cm_max=PxpGPlotPosition(phys_max);

  PxpDrawRectangle(cm_min.x,cm_min.y,cm_max.x,cm_max.y,1);

  return;
}

void PxpGGrid(phys_min,phys_max,interval_x,interval_y)
PxpPoint phys_min,phys_max; /* <in> left and right position (phys unit); min=left,bottom; max=right,top */
double interval_x,interval_y; /* <in> tickmark interval (phys unit) */
/*
plot grid in physics unit
1997/10/21 T.Handa ver.1.00
2000/ 2/14 T.Handa ver.1.13 (revise) shrink
*/
{
  PxpGGridInset(phys_min,phys_max,phys_min,phys_max,interval_x,interval_y);

  return;
}

void PxpGGridInset(phys_min,phys_max,phys_start,phys_stop,interval_x,interval_y)
PxpPoint phys_min,phys_max; /* <in> left and right position (phys unit); min=left,bottom; max=right,top */
PxpPoint phys_start,phys_stop;
double interval_x,interval_y;
/*
plot grid in physics unit
if gridding area is smaller than the total ploting area
1997/10/22 T.Handa ver.1.00
1999/2/10 T.Handa ver.1.01 (debug) sign of dy
2000/ 2/15 T.Handa ver.1.14 (modify) for single and zero tick mark
*/
{
  PxpPoint cm_min,cm_max;
  PxpPoint cm_start,cm_stop;
  double int_x,int_y;
  int ii,jj,n_line,m_line;
  double xx,yy;
  double dx,dy;

  cm_min=PxpGPlotPosition(phys_min);
  cm_max=PxpGPlotPosition(phys_max);
  cm_start=PxpGPlotPosition(phys_start);
  cm_stop=PxpGPlotPosition(phys_stop);

  n_line=(int)rint((phys_stop.x-phys_start.x)/interval_x)+1;
  if(n_line==1) {
    xx=cm_start.x;
    PxpDrawLine(xx,cm_min.y,xx,cm_max.y);	
  } else {
    dx=(cm_stop.x-cm_start.x)/(double)(n_line-1);
    for(ii=0;ii<n_line;ii++) {
      xx=dx*(double)ii+cm_start.x;
      PxpDrawLine(xx,cm_min.y,xx,cm_max.y);	
    }
  }

  m_line=(int)rint((phys_stop.y-phys_start.y)/interval_y)+1;
  if(m_line==1) {
    yy=cm_start.y;
    PxpDrawLine(cm_min.x,yy,cm_max.x,yy);	
  } else {
    dy=(cm_stop.y-cm_start.y)/(double)(m_line-1);
    for(jj=0;jj<m_line;jj++) {
      yy=dy*(double)jj+cm_start.y;
      PxpDrawLine(cm_min.x,yy,cm_max.x,yy);	
    }
  }

  return;
}

void PxpGTickMark(phys_min,phys_max,interval_x,interval_y,tick_length)
PxpPoint phys_min,phys_max; /* <in> left and right position (phys unit); min=left,bottom; max=right,top */
double interval_x,interval_y; /* <in> tickmark interval (phys unit) */
double tick_length; /* <in> tickmark length (plot unit) */
/*
plot tickmarks at proper position in physics unit.
1997/10/22 T.Handa ver.1.00
1999/10/ 2 F.Yamamoto ver.1.12 (debug)
2000/ 2/14 T.Handa ver.1.13 (revise) shrink
*/
{
  PxpGTickMarkInset(phys_min,phys_max,phys_min,phys_max,interval_x,interval_y,tick_length);

  return;
}

void PxpGTickMarkInset(phys_min,phys_max,phys_start,phys_stop,interval_x,interval_y,tick_length)
PxpPoint phys_min,phys_max; /* <in> left and right position (phys unit); min=left,bottom; max=right,top */
PxpPoint phys_start,phys_stop;	/* <in> tick mark start/stop value; start=left,bottom; stop=right,top */
double interval_x,interval_y; /* <in> tickmark interval (phys unit) */
double tick_length; /* <in> tickmark length (plot unit) */
/*
plot tickmarks at proper position in physics unit.
if gridding area is smaller than the total ploting area
1997/10/22 T.Handa ver.1.00
1999/10/ 2 F.Yamamoto ver.1.12 (debug)
2000/ 2/15 T.Handa ver.1.14 (modify) for single and zero tick mark
*/
{
  PxpPoint cm_min,cm_max;
  PxpPoint cm_start,cm_stop;
  double int_x,int_y;
  int ii,jj,n_line,m_line;
  double xx,yy;
  double dx,dy;

  cm_min=PxpGPlotPosition(phys_min);
  cm_max=PxpGPlotPosition(phys_max);
  cm_start=PxpGPlotPosition(phys_start);
  cm_stop=PxpGPlotPosition(phys_stop);

  n_line=(int)rint((phys_stop.x-phys_start.x)/interval_x)+1;
  if(n_line==1) {
    xx=cm_start.x;
    PxpDrawLine(xx,cm_min.y,xx,cm_min.y+tick_length);	
    PxpDrawLine(xx,cm_max.y-tick_length,xx,cm_max.y);	
  } else {
    dx=(cm_stop.x-cm_start.x)/(double)(n_line-1);
    for(ii=0;ii<n_line;ii++) {
      xx=dx*(double)ii+cm_start.x;
      PxpDrawLine(xx,cm_min.y,xx,cm_min.y+tick_length);	
      PxpDrawLine(xx,cm_max.y-tick_length,xx,cm_max.y);	
    }
  }

  m_line=(int)rint((phys_stop.y-phys_start.y)/interval_y)+1;
  if(m_line==1) {
    yy=cm_start.y;
    PxpDrawLine(cm_min.x,yy,cm_min.x+tick_length,yy);	
    PxpDrawLine(cm_max.x-tick_length,yy,cm_max.x,yy);	
  } else {
    dy=(cm_stop.y-cm_start.y)/(double)(m_line-1);
    for(jj=0;jj<m_line;jj++) {
      yy=dy*(double)jj+cm_start.y;
      PxpDrawLine(cm_min.x,yy,cm_min.x+tick_length,yy);	
      PxpDrawLine(cm_max.x-tick_length,yy,cm_max.x,yy);	
    }
  }

  return;
}

void PxpGLabel(phys_min,phys_max,interval_x,interval_y,format_x,format_y)
PxpPoint phys_min,phys_max; /* <in> left and right position (phys unit); min=left,bottom; max=right,top */
double interval_x,interval_y; /* <in> tickmark interval (phys unit) */
char format_x[],format_y[];
/*
plot labels by physics unit
1997/10/23 T.Handa ver.1.00
1999/10/ 2 F.Yamamoto ver.1.12 (debug)
2000/ 2/14 T.Handa ver.1.13 (revise) shrink
*/
{
  PxpGLabelInset(phys_min,phys_max,phys_min,phys_max,interval_x,interval_y,format_x,format_y);

  return;
}

void PxpGLabelInset(phys_min,phys_max,phys_start,phys_stop,interval_x,interval_y,format_x,format_y)
PxpPoint phys_min,phys_max; /* <in> left and right position (phys unit); min=left,bottom; max=right,top */
PxpPoint phys_start,phys_stop;	/* <in> label start/stop value; start=left,bottom; stop=right,top */
double interval_x,interval_y; /* <in> tickmark interval (phys unit) */
char format_x[],format_y[];
/*
plot labels by physics unit
if the labeling area is smaller than the total ploting area
1997/12/10 T.Handa ver.1.00
1999/10/ 2 F.Yamamoto ver.1.12 (debug)
2000/ 2/15 T.Handa ver.1.14 (modify) for single and zero tick mark
*/
{
  PxpPoint cm_min,cm_max;
  PxpPoint cm_start,cm_stop;
  double int_x,int_y;
  int ii,jj,n_line,m_line;
  double xx,yy;
  double dx,dy;
  double phys_x,phys_y;
  char buffer[BUFFER_SIZE];

  cm_min=PxpGPlotPosition(phys_min);
  cm_max=PxpGPlotPosition(phys_max);
  cm_start=PxpGPlotPosition(phys_start);
  cm_stop=PxpGPlotPosition(phys_stop);

  n_line=(int)rint((phys_stop.x-phys_start.x)/interval_x)+1;
  if(n_line==1) {
    phys_x=phys_start.x;
    if(format_x==NULL || format_x[0]==NULL)
      sprintf(buffer,"%f",phys_x);
    else
      sprintf(buffer,format_x,phys_x);
    xx=cm_start.x;
    PxpDrawJustifiedString(xx,cm_min.y,buffer,"center","hanging");	
  } else {
    dx=(cm_stop.x-cm_start.x)/(double)(n_line-1);
    for(ii=0;ii<n_line;ii++) {
      phys_x=phys_start.x+interval_x*(double)ii;
      if(format_x==NULL || format_x[0]==NULL)
	sprintf(buffer,"%f",phys_x);
      else
	sprintf(buffer,format_x,phys_x);
      xx=dx*(double)ii+cm_start.x;
      PxpDrawJustifiedString(xx,cm_min.y,buffer,"center","hanging");	
    }
  }
  
  m_line=(int)rint((phys_stop.y-phys_start.y)/interval_y)+1;
  if(m_line==1) {
    phys_y=phys_start.y;
    if(format_y==NULL || format_y[0]==NULL)
      sprintf(buffer,"%f",phys_y);
    else
      sprintf(buffer,format_y,phys_y);
    yy=cm_start.y;
    PxpDrawJustifiedString(cm_min.x-0.1,yy,buffer,"right","center");	
  } else {
    dy=(cm_stop.y-cm_start.y)/(double)(m_line-1);
    for(jj=0;jj<m_line;jj++) {
      phys_y=phys_start.y+interval_y*(double)jj;
      if(format_y==NULL || format_y[0]==NULL)
	sprintf(buffer,"%f",phys_y);
      else
	sprintf(buffer,format_y,phys_y);
      yy=dy*(double)jj+cm_start.y;
      PxpDrawJustifiedString(cm_min.x-0.1,yy,buffer,"right","center");	
    }
  }

  return;
}

void PxpGLine(x1,y1,x2,y2)
double x1,y1,x2,y2;
/*
draw a line in physics unit
1997/12/10 T.Handa ver.1.00
*/
{
  PxpPoint phys1,phys2,plot1,plot2;

  phys1.x=x1;
  phys1.y=y1;
  phys2.x=x2;
  phys2.y=y2;

  plot1=PxpGPlotPosition(phys1);
  plot2=PxpGPlotPosition(phys2);
  PxpDrawLine(plot1.x,plot1.y,plot2.x,plot2.y);

  return;
}

void PxpGLines(pt,n_pt)
PxpPoint *pt;
int n_pt;
/*
draw connected lines in physics unit
1997/10/21 T.Handa ver.1.00
*/
{
  PxpPoint *pos_list;
  int	ii;

  pos_list=(PxpPoint *)malloc(sizeof(PxpPoint)*n_pt);
  
  for(ii=0;ii<n_pt;ii++) {
    pos_list[ii]=PxpGPlotPosition(pt[ii]);
  }
  PxpDrawLines(pos_list,n_pt);
  free(pos_list);

  return;
}

void PxpGMark(pt_x,pt_y,type,size)
double pt_x,pt_y;
char *type;
double size;
/*
draw a centered mark in physics unit
1997/10/23 T.Handa ver.1.00
*/
{
  PxpPoint phys,position;

  phys.x=pt_x; phys.y=pt_y;
  position=PxpGPlotPosition(phys);
  PxpDrawMark(position.x,position.y,type,size);

  return;
}

void PxpGErrorBar(pt_x,pt_y,error,size)
double pt_x,pt_y;
double error;
double size;
/*
draw a centered error bar in physics unit
error in vertical direction
1997/10/23 T.Handa ver.1.00
*/
{
  PxpPoint phys,position,half_span;
  double xx,y1,y2,x1,x2;

  phys.x=pt_x; phys.y=pt_y;
  position=PxpGPlotPosition(phys);
  phys.x=0.0; phys.y=error;
  half_span=PxpGPlotSpan(phys);
  xx=position.x;
  y1=position.y+half_span.y;
  y2=position.y-half_span.y;
  x1=xx-size*0.5; x2=xx+size*0.5;
  PxpDrawLine(xx,y1,xx,y2);
  PxpDrawLine(x1,y1,x2,y1);
  PxpDrawLine(x1,y2,x2,y2);

  return;
}

void PxpGErrorBarX(pt_x,pt_y,error,size)
double pt_x,pt_y;
double error;
double size;
/*
draw a centered error bar in physics unit
error in horizontal direction
1997/10/23 T.Handa ver.1.00
*/
{
  PxpPoint phys,position,half_span;
  double yy,x1,x2,y1,y2;

  phys.x=pt_x; phys.y=pt_y;
  position=PxpGPlotPosition(phys);
  phys.x=error; phys.y=0.0;
  half_span=PxpGPlotSpan(phys);
  yy=position.y;
  x1=position.x+half_span.x;
  x2=position.x-half_span.x;
  y1=yy-size*0.5; y2=yy+size*0.5;
  PxpDrawLine(x1,yy,x2,yy);
  PxpDrawLine(x1,y1,x1,y2);
  PxpDrawLine(x2,y1,x2,y2);

  return;
}

/* assistant for graph plotting */

void PxpGCeilingRange(orig_range,ceil_range,n_step,step)
/*
estimate properly rounded range
which encloses the original range,
and proper steps for division
  Effecitve to estimate tickmarks and labels on axis

1997/9/7 T.Handa (based upon STAR_AutoScale(); 1992/9/7)
2000/2/15 T.Handa (debug) for negative span, over 0.0
*/
PxpRange orig_range,	/* <in>  original range */
	 *ceil_range;	/* <out> rounded range (ceiled) */
int	*n_step;	/* <out> number of steps, always positive */
double	*step;		/* <out> step interval */
{
  double span,abs_span;
  double abs_step;
  double abs_min,abs_max;

  span=orig_range.max-orig_range.min;
  abs_span=fabs(span);

  if(span==0.0) { /* no span */
    (*ceil_range).min=orig_range.min;
    (*ceil_range).max=orig_range.max;
    *n_step=1;
    *step=0.0;
    return;
  } else if(orig_range.max * orig_range.min <0.0) { /* range is beyond 0.0 */
    abs_min=fabs(orig_range.min);
    abs_max=fabs(orig_range.max);
    if(abs_min>abs_max)
      abs_span=abs_min;
    else
      abs_span=abs_max;
  }
  abs_step=pow(10.0, floor( log10(abs_span) ))/10.0;
  *n_step=floor(abs_span/abs_step);

  if((*n_step)>20) {
    abs_step=abs_step*5.0;
    *n_step=floor(abs_span/abs_step);
  }
  if((*n_step)>10) {
    abs_step=abs_step*2.0;
    *n_step=floor(abs_span/abs_step);
  }

  if((*ceil_range).min>0.0)
    (*ceil_range).min=floor(orig_range.min/abs_step) * abs_step;
  else
    (*ceil_range).min= -ceil(-orig_range.min/abs_step) * abs_step;

  if((*ceil_range).max>0.0)
    (*ceil_range).max=ceil(orig_range.max/abs_step) * abs_step;
  else
    (*ceil_range).max= -floor(-orig_range.max/abs_step) * abs_step;

  *n_step=floor(fabs((*ceil_range).max-(*ceil_range).min)/abs_step)+1;

  if(span>0.0)
    *step= abs_step;
  else
    *step= -abs_step;

  return;
}

PxpRange PxpGEnclosureRange2InteriorRange(enclosure,range,xq)
PxpRange enclosure; /* <in> enclosure box */
PxpRange range; /* <in> standard box */
double xq; /* <in> quanta */
/*
convert enclosure to interior
2000/2/16 T.Handa
*/
{
  PxpRange interior;
  PxpRange std_range,inter,encl;
  double quantum;

/* sign justify */
  if(range.max>range.min) {
    std_range.min=range.min;
    std_range.max=range.max;
  } else {
    std_range.min=range.max;
    std_range.max=range.min;
  }
  if(enclosure.max>enclosure.min) {
    encl.min=enclosure.min;
    encl.max=enclosure.max;
  } else {
    encl.min=enclosure.max;
    encl.max=enclosure.min;
  }

  quantum=fabs(xq);

/* upper enclosure --> upper interior */
  if(encl.max>std_range.max)
    inter.max=encl.max-ceil((encl.max-std_range.max)/quantum)*quantum;
  else if(encl.max==std_range.max)
    inter.max=encl.max;
  else
    inter.max=encl.max+floor((std_range.max-encl.max)/quantum)*quantum;

/* lower enclosure --> lower interior */
  if(encl.min<std_range.min)
    inter.min=encl.min+ceil((std_range.min-encl.min)/quantum)*quantum;
  else if(encl.min==std_range.min)
    inter.min=encl.min;
  else
    inter.min=encl.max-floor((encl.min-std_range.min)/quantum)*quantum;

/* re-adjust upper and lower */
  if(enclosure.max>enclosure.min) {
    interior.min=inter.min;
    interior.max=inter.max;
  } else {
    interior.max=inter.min;
    interior.min=inter.max;
  }

  return interior;
}

void PxpGLabelFormat(label_format,range,step,sign)
char *label_format;	/* <out> strings for best label */
PxpRange range;	/* <in> label range */
double step;	/* <in> label interval */
char sign;	/* <in> number with sign or not '+','-',' ' */
/*
create the best format string for labeling
label_format must be allocated before.
1997/12/10 T.Handa
*/
{
  double max,min;
  int max_digit,min_digit,step_digit;
  int max_sign,min_sign;
  int max_order;

/* estimate the number of digits for range.max and range.min */
  if(range.max>0) {
    max_sign= 1;
    max=range.max;
  } else {
    max_sign= -1;
    max= -range.max;
  }

  if(range.min>0) {
    min_sign= 1;
    min=range.min;
  } else {
    min_sign= -1;
    min= -range.min;
  }

/* range has integer part */
  if(max==0.0) {
    max_digit=1;
  } else if(max>=1.0) {
    max_digit=(int)(floor(log10(max)))+1;
  } else {
    max_digit=(int)(floor(-log10(max)))+3;
  }
  if(min==0.0) {
    min_digit=1;
  } else if(min>=1.0) {
    min_digit=(int)(floor(log10(min)))+1;
  } else {
    min_digit=(int)(floor(-log10(min)))+3;
  }

  if(max_digit>min_digit) max_order=max_digit;
  else max_order=min_digit;

/* integer format */
  if(fabs(step)>=1.0 || step==0.0) {
    switch(sign) {
    case '+': case '-':
      sprintf(label_format,"%%%c%d.0f",sign,max_order);
      break;
    default:
      sprintf(label_format,"%%%d.0f",max_order);
      break;
    }
    return;
  }

/* effective fractions of step */
  if(max<1.0) max_digit=1;
  if(min<1.0) min_digit=1;
  if(max_digit>min_digit) max_order=max_digit;
  else max_order=min_digit;

  step_digit=(int)(floor(-log10(fabs(step))))+1;
  max_order=max_order+step_digit+1;
  switch(sign) {
  case '+': case '-':
    sprintf(label_format,"%%%c%d.%df",sign,max_order,step_digit);
  default:
    sprintf(label_format,"%%%d.%df",max_order,step_digit);
  }
  return;
}
