/*
Pxp -- Plot for X11 window and/or Postscript printer

Library for X11 display and/or PostScript printer
1997/10/26 T.Handa ver.1.00
1997/12/03 T.Handa ver.1.02 for CISC-News
1999/10/ 2 F.Yamamoto ver.1.12 (debug) PxpDrawJustifiedText();
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include <Xm/DrawingA.h>
#include <Xm/MainW.h>
#include <Xm/RowColumn.h>
#include <Xm/CascadeB.h>
#include <Xm/PushBG.h>

#include <Pxxlib.h>

#include <Ppslib.h>
#include <Pxplib.h>

#define	BUFFER_SIZE	256

static int PxpView_output=X11WIN;
static int	PxpView_fontsize=14;


/* administration functions: get global parameters */
void PxpGetOutputMode(output_mode)
int *output_mode;
{
  *output_mode=PxpView_output;

  return;
}

/* administration functions: put global parameters */
void PxpPutOutputMode(output_mode)
int output_mode;
{
  PxpView_output=output_mode;

  return;
}

void PxpSetPrinterName(prt_name)
char prt_name[];
/*
output printer name
1997/10/13 T.Handa ver.1.00
*/
{
  switch(PxpView_output) {
  case X11WIN:
    PxxSetPrinterName(prt_name);
    break;
  case POSTSCRIPT:
    PpsSetPrinterName(prt_name);
  }

  return;
}
void	PxpGetScale(x,y)
double *x,*y;
/*
get view scale
1997/10/21 T.Handa ver.1.00
*/
{
  switch(PxpView_output) {
  case X11WIN:
    PxxGetScale(x,y);
    break;
  case POSTSCRIPT:
    PpsGetScale(x,y);
  }

  return;
}
void	PxpGetOrigin(x,y)
double *x,*y;
/*
get view origin
1997/10/21 T.Handa ver.1.00
*/
{
  switch(PxpView_output) {
  case X11WIN:
    PxxGetOrigin(x,y);
    break;
  case POSTSCRIPT:
    PpsGetOrigin(x,y);
  }

  return;
}
void	PxpPutScale(x,y)
double x,y;
/*
set view scale
1997/10/21 T.Handa ver.1.00
*/
{
  switch(PxpView_output) {
  case X11WIN:
    PxxPutScale(x,y);
    break;
  case POSTSCRIPT:
    PpsPutScale(x,y);
  }

  return;
}
void	PxpPutOrigin(x,y)
double x,y;
/*
set view origin
1997/10/21 T.Handa ver.1.00
*/
{
  switch(PxpView_output) {
  case X11WIN:
    PxxPutOrigin(x,y);
    break;
  case POSTSCRIPT:
    PpsPutOrigin(x,y);
  }

  return;
}

/* draw graphics */

void PxpSetColor(color)
char color[];
/*
set line color
1997/10/12 T.Handa ver.1.00
*/
{
  switch(PxpView_output) {
  case X11WIN:
    PxxSetColor(color);
    break;
  case POSTSCRIPT:
    PpsSetColor(color);
  }

  return;
}

void PxpSetGray(density)
double	density;
/*
set line gray
1997/10/12 T.Handa ver.1.00
*/
{
  switch(PxpView_output) {
  case X11WIN:
    PxxSetGray(density);
    break;
  case POSTSCRIPT:
    PpsSetGray(density);
    break;
  }

  return;
}

void PxpSetLineWidth(linewidth)
double linewidth;
/*
set line width
1997/10/12 T.Handa ver.1.00
*/
{
  switch(PxpView_output) {
  case X11WIN:
    PxxSetLineWidth(linewidth);
    break;
  case POSTSCRIPT:
    PpsSetLineWidth(linewidth);
  }

  return;
}
void PxpSetLinePattern(line_patt,n_patt,phase)
double line_patt[];
int n_patt;
double phase;
/*
set line patern
1997/10/14 T.Handa ver.1.00
*/
{
  switch(PxpView_output) {
  case X11WIN:
    PxxSetLinePattern(line_patt,n_patt,phase);
    break;
  case POSTSCRIPT:
    PpsSetLinePattern(line_patt,n_patt,phase);
  }

  return;
}
void PxpSetLineType(type_name)
char type_name[];
/*
set line type
1997/10/16 T.Handa ver.1.00
*/
{
  double patt_list[64],phase;
  int nn;

  phase=0.0;
  if(strcmp(type_name,"solid")==0) {
    patt_list[0]=1.0;
    nn=1;
  } else if(strcmp(type_name,"dashed")==0 ||strcmp(type_name,"dash")==0) {
    patt_list[0]=0.3;
    patt_list[1]=0.2;
    nn=2;
    phase=0.15;
  } else if(strcmp(type_name,"dotted")==0 ||strcmp(type_name,"dot")==0) {
    patt_list[0]=0.05;
    patt_list[1]=0.20;
    nn=2;
  } else if(strcmp(type_name,"dotted-dashed")==0 ||strcmp(type_name,"dotted dashed")==0 ||strcmp(type_name,"dotted dash")==0 ||strcmp(type_name,"dot dash")==0) {
    patt_list[0]=0.3;
    patt_list[1]=0.20;
    patt_list[2]=0.05;
    patt_list[3]=0.20;
    nn=4;
    phase=0.15;
  } else {		
    patt_list[0]=1.0;
    nn=1;
  }

  PxpSetLinePattern(patt_list,nn,phase);

  return;
}

void PxpDrawLine(x1,y1,x2,y2)
double x1,y1,x2,y2;
/*
draw a straight line
1997/10/12 T.Handa ver.1.00
*/
{
  switch(PxpView_output) {
  case X11WIN:
    PxxDrawLine(x1,y1,x2,y2);
    break;
  case POSTSCRIPT:
    PpsDrawLine(x1,y1,x2,y2);
  }

  return;
}
void PxpDrawLines(pos_list,n_pt)
PxpPoint pos_list[];
int	n_pt;
/*
draw line segments
1997/10/19 T.Handa ver.1.00
*/
{
  switch(PxpView_output) {
  case X11WIN:
    PxxDrawLines((PxxPoint *)pos_list,n_pt);
    break;
  case POSTSCRIPT:
    PpsDrawLines((PpsPoint *)pos_list,n_pt);
  }

  return;
}

void PxpDrawRectangle(x1,y1,x2,y2,fill)
double x1,y1,x2,y2;
int	fill;
/*
draw a rectangle
1997/10/12 T.Handa ver.1.00
*/
{
  switch(PxpView_output) {
  case X11WIN:
    PxxDrawRectangle(x1,y1,x2,y2,fill);
    break;
  case POSTSCRIPT:
    PpsDrawRectangle(x1,y1,x2,y2,fill);
  }

  return;
}
void PxpDrawPolygon(pos_list,n_pt,fill)
PxpPoint pos_list[];
int	n_pt;
int	fill;
/*
draw a polygon
1997/10/19 T.Handa ver.1.00
*/
{
  switch(PxpView_output) {
  case X11WIN:
    PxxDrawPolygon((PxxPoint *)pos_list,n_pt,fill);
    break;
  case POSTSCRIPT:
    PpsDrawPolygon((PpsPoint *)pos_list,n_pt,fill);
  }

  return;
}

void PxpDrawArc(xx,yy,radius,start,theta,fill)
double xx,yy,radius;
double start,theta;
int fill;
/*
draw an arc
1997/10/12 T.Handa ver.1.00
*/
{
  switch(PxpView_output) {
  case X11WIN:
    PxxDrawArc(xx,yy,radius,start,theta,fill);
    break;
  case POSTSCRIPT:
    PpsDrawArc(xx,yy,radius,start,theta,fill);
  }

  return;
}

void PxpDrawSector(xx,yy,radius,start,theta,fill)
double xx,yy,radius;
double start,theta;
int	fill;
/*
draw a sector
1997/10/12 T.Handa ver.1.00
*/
{
  switch(PxpView_output) {
  case X11WIN:
    PxxDrawSector(xx,yy,radius,start,theta,fill);
    break;
  case POSTSCRIPT:
    PpsDrawSector(xx,yy,radius,start,theta,fill);
  }

  return;
}

void PxpDrawMark(xx,yy,type,size)
double xx,yy;
char type[];
double size;
/*
draw a mark located by center position
size assigned by
circle=diameter,
square&diamond&trangle=length of a segment,
others=diameter of the outer envelope circle
1997/10/24 T.Handa ver.1.00
*/
{
  double x1,x2,x3,x4,y1,y2,y3,y4;
  PxpPoint point[5];
  int fill;

  if(strcmp(type,"circle")==0) {
    PxpDrawArc(xx,yy,size*0.5,0.0,360.0,0);
  } else if(strcmp(type,"filled circle")==0 || strcmp(type,"dot")==0) {
    PxpDrawArc(xx,yy,size*0.5,0.0,360.0,1);
  } else if(strcmp(type,"triangle")==0 || strcmp(type,"filled triangle")==0) {
    x1=xx-size*0.5;
    x2=xx+size*0.5;
    y1=yy-size*0.2886751346;
    y2=yy+size*0.5773502692;
    point[0].x=xx; point[0].y=y2;
    point[1].x=x1; point[1].y=y1;
    point[2].x=x2; point[2].y=y1;
    if(type[0]=='f') PxpDrawPolygon(point,3,1);
    else PxpDrawPolygon(point,3,0);
  } else if(strcmp(type,"upside-down triangle")==0 || strcmp(type,"filled upside-down triangle")==0) {
    x1=xx-size*0.5;
    x2=xx+size*0.5;
    y1=yy+size*0.2886751346;
    y2=yy-size*0.5773502692;
    point[0].x=x1; point[0].y=y1;
    point[1].x=xx; point[1].y=y2;
    point[2].x=x2; point[2].y=y1;
    if(type[0]=='f') PxpDrawPolygon(point,3,1);
    else PxpDrawPolygon(point,3,0);
  } else if(strcmp(type,"square")==0 || strcmp(type,"filled square")==0) {
    x1=xx-size*0.5;
    x2=xx+size*0.5;
    y1=yy-size*0.5;
    y2=yy+size*0.5;
    if(type[0]=='f') PxpDrawRectangle(x1,y1,x2,y2,1);
    else PxpDrawRectangle(x1,y1,x2,y2,0);
  } else if(strcmp(type,"diamond")==0 || strcmp(type,"filled diamond")==0) {
    point[0].x=xx; point[0].y=yy+size*0.7071067812;
    point[1].x=xx-size*0.7071067812; point[1].y=yy;
    point[2].x=xx; point[2].y=yy-size*0.7071067812;
    point[3].x=xx+size*0.7071067812; point[3].y=yy;
    if(type[0]=='f') PxpDrawPolygon(point,4,1);
    else PxpDrawPolygon(point,4,0);
  } else if(strcmp(type,"cross")==0 || strcasecmp(type,"x")==0) {
    x1=xx-size*0.3535533906;	/* 1/sqrt(2)/2 */
    x2=xx+size*0.3535533906;
    y1=yy-size*0.3535533906;
    y2=yy+size*0.3535533906;
    PxpDrawLine(x1,y1,x2,y2);
    PxpDrawLine(x1,y2,x2,y1);
  } else if(strcmp(type,"plus")==0 || strcmp(type,"+")==0) {
    x1=xx-size*0.5;
    x2=xx+size*0.5;
    y1=yy-size*0.5;
    y2=yy+size*0.5;
    PxpDrawLine(xx,y1,xx,y2);
    PxpDrawLine(x1,yy,x2,yy);
  } else if(strcmp(type,"star")==0 || strcmp(type,"filled star")==0) {
    x1=xx-size*0.4755282582;
    x2=xx-size*0.2938926262;
    x3=xx+size*0.2938926262;
    x4=xx+size*0.4755282582;
    y1=yy+size*0.5;
    y2=yy+size*0.1545084972;
    y3=yy-size*0.4045084972;
    point[0].x=xx; point[0].y=y1;
    point[1].x=x2; point[1].y=y3;
    point[2].x=x4; point[2].y=y2;
    point[3].x=x1; point[3].y=y2;
    point[4].x=x3; point[4].y=y3;
    if(type[0]=='f') PxpDrawPolygon(point,5,1);
    else PxpDrawPolygon(point,5,0);
  } else if(strcmp(type,"David's star")==0 || strcmp(type,"filled David's star")==0 || strcmp(type,"asterisk")==0 || strcmp(type,"*")==0) {
    x1=xx-size*0.4330127019;
    x2=xx+size*0.4330127019;
    y1=yy+size*0.5;
    y2=yy+size*0.25;
    y3=yy-size*0.25;
    y4=yy-size*0.5;
    if(type[0]=='a' || type[0]=='*') {
      PxpDrawLine(xx,y1,xx,y4);
      PxpDrawLine(x1,y2,x2,y3);
      PxpDrawLine(x1,y3,x2,y2);
    } else {
      if(type[0]=='f')
	fill=1;
      else
	fill=NULL;
      point[0].x=xx; point[0].y=y1;
      point[1].x=x1; point[1].y=y3;
      point[2].x=x2; point[2].y=y3;
      PxpDrawPolygon(point,3,fill);
      point[0].x=x1; point[0].y=y2;
      point[1].x=xx; point[1].y=y4;
      point[2].x=x2; point[2].y=y2;
      PxpDrawPolygon(point,3,fill);
    }
  }

  return;
}

/* plot a character */
#define N_FONTS 128

void PxpSetFont(font_name,size)
char font_name[];
short size;
/*
set font and its size
1997/10/12 T.Handa ver.1.00
*/
{
  static char x_font[N_FONTS][6];
  static short x_font_size[N_FONTS];
  static int n_Xfont=0;
  FILE	*fp;
  char	buffer[BUFFER_SIZE],*token;
  int	ii,l_str;
  int	proper_size;

  PxpView_fontsize=(int)size;

  if(n_Xfont==0) {
    token=getenv("PXP_XFONTLIST");
    if(token==NULL)
      fp=fopen("/usr/lib/X11/fonts/misc/fonts.alias","r");
    else
      fp=fopen(token,"r");
    for(n_Xfont=0;n_Xfont<N_FONTS;) {
      if(fgets(buffer,BUFFER_SIZE,fp)==NULL) break;
      if(feof(fp)!=NULL) break;
      l_str=strlen(buffer);
      if(buffer[l_str-1]!='\n') break;
      token=strtok(buffer," ,\t");
      if(token==NULL) break;
      if(strlen(token)>2 && strlen(token)<6 && (token[1]=='x' || token[2]=='x')) {
	strcpy(x_font[n_Xfont],token);
	n_Xfont++;
      }
    }
    fclose(fp);
    for(ii=0;ii<n_Xfont;ii++) {
      if(x_font[ii][1]=='x')
	x_font_size[ii]=(short)atoi(&(x_font[ii][2]));
      else if(x_font[ii][2]=='x')
	x_font_size[ii]=(short)atoi(&(x_font[ii][3]));
      else
	x_font_size[ii]= -1;
    }
  }

  if(strcmp(font_name,"PxpDefault")==0 || font_name[0]==NULL) {
    switch(PxpView_output) {
    case X11WIN:
      proper_size=0;
      for(ii=0;ii<n_Xfont;ii++) {
	if(size==x_font_size[ii]) {
	  PxxSetFont(x_font[ii]);
	  proper_size=1;
	}
      }
      if(proper_size==0) PxxSetFont("7x14");
      break;
    case POSTSCRIPT:
      PpsSetFont("Courier",size);
    }
  } else if(strcmp(font_name,"Times")==0) {
    switch(PxpView_output) {
    case X11WIN:
      if(size<10) PxxSetFont("times8");
      else if(size<12) PxxSetFont("times10");
      else if(size<16) PxxSetFont("times12");
      else if(size<22) PxxSetFont("times16");
      else PxxSetFont("times22");
      break;
    case POSTSCRIPT:
      PpsSetFont("Times-Roman",size);
    }
  } else if(strcmp(font_name,"Times-Bold")==0) {
    switch(PxpView_output) {
    case X11WIN:
      if(size<10) PxxSetFont("times8b");
      else if(size<12) PxxSetFont("times10b");
      else if(size<16) PxxSetFont("times12b");
      else if(size<22) PxxSetFont("times16b");
      else PxxSetFont("times22b");
      break;
    case POSTSCRIPT:
      PpsSetFont("Times-Bold",size);
    }
  } else if(strcmp(font_name,"Times-Italic")==0) {
    switch(PxpView_output) {
    case X11WIN:
      if(size<10) PxxSetFont("times8i");
      else if(size<12) PxxSetFont("times10i");
      else if(size<16) PxxSetFont("times12i");
      else if(size<22) PxxSetFont("times16i");
      else PxxSetFont("times22i");
      break;
    case POSTSCRIPT:
      PpsSetFont("Times-Italic",size);
    }
  } else if(strcmp(font_name,"Times-BoldItalic")==0) {
    switch(PxpView_output) {
    case X11WIN:
      if(size<10) PxxSetFont("times8bi");
      else if(size<12) PxxSetFont("times10bi");
      else if(size<16) PxxSetFont("times12bi");
      else if(size<22) PxxSetFont("times16bi");
      else PxxSetFont("times22bi");
      break;
    case POSTSCRIPT:
      PpsSetFont("Times-BoldItalic",size);
    }
  } else if(strcmp(font_name,"Helvetica")==0) {
    switch(PxpView_output) {
    case X11WIN:
      if(size<10) PxxSetFont("helvetica8");
      else if(size<12) PxxSetFont("helvetica10");
      else if(size<16) PxxSetFont("helvetica12");
      else if(size<22) PxxSetFont("helvetica16");
      else PxxSetFont("helvetica22");
      break;
    case POSTSCRIPT:
      PpsSetFont("Helvetica",size);
    }
  } else if(strcmp(font_name,"Helvetica-Bold")==0) {
    switch(PxpView_output) {
    case X11WIN:
      if(size<10) PxxSetFont("helvetica8b");
      else if(size<12) PxxSetFont("helvetica10b");
      else if(size<16) PxxSetFont("helvetica12b");
      else if(size<22) PxxSetFont("helvetica16b");
      else PxxSetFont("helvetica22b");
      break;
    case POSTSCRIPT:
      PpsSetFont("Helvetica-Bold",size);
    }
  } else if(strcmp(font_name,"Helvetica-Italic")==0) {
    switch(PxpView_output) {
    case X11WIN:
      if(size<10) PxxSetFont("helvetica8i");
      else if(size<12) PxxSetFont("helvetica10i");
      else if(size<16) PxxSetFont("helvetica12i");
      else if(size<22) PxxSetFont("helvetica16i");
      else PxxSetFont("helvetica22i");
      break;
    case POSTSCRIPT:
      PpsSetFont("Helvetica-Oblique",size);
    }
  } else if(strcmp(font_name,"Helvetica-BoldItalic")==0) {
    switch(PxpView_output) {
    case X11WIN:
      if(size<10) PxxSetFont("helvetica8bi");
      else if(size<12) PxxSetFont("helvetica10bi");
      else if(size<16) PxxSetFont("helvetica12bi");
      else if(size<22) PxxSetFont("helvetica16bi");
      else PxxSetFont("helvetica22bi");
      break;
    case POSTSCRIPT:
      PpsSetFont("Helvetica-BoldOblique",size);
    }
  } else if(strcmp(font_name,"Courier")==0) {
    switch(PxpView_output) {
    case X11WIN:
      if(size<10) PxxSetFont("courier8f");
      else if(size<12) PxxSetFont("courier10f");
      else PxxSetFont("courier12f");
      break;
    case POSTSCRIPT:
      PpsSetFont("Courier",size);
    }
  } else if(strcmp(font_name,"Courier-Bold")==0) {
    switch(PxpView_output) {
    case X11WIN:
      if(size<10) PxxSetFont("courier8bf");
      else if(size<12) PxxSetFont("courier10bf");
      else PxxSetFont("courier12bf");
      break;
    case POSTSCRIPT:
      PpsSetFont("Courier-Bold",size);
    }
  } else if(strcmp(font_name,"Courier-Italic")==0) {
    switch(PxpView_output) {
    case X11WIN:
      if(size<10) PxxSetFont("courier8if");
      else PxxSetFont("courier10if");
      break;
    case POSTSCRIPT:
      PpsSetFont("Courier-Oblique",size);
    }
  } else if(strcmp(font_name,"Courier-BoldItalic")==0) {
    switch(PxpView_output) {
    case X11WIN:
      if(size<10) PxxSetFont("courier8bif");
      else PxxSetFont("courier10bif");
      break;
    case POSTSCRIPT:
      PpsSetFont("Courier-BoldOblique",size);
    }
  } else if(strcmp(font_name,"Symbol")==0) {
    switch(PxpView_output) {
    case X11WIN:
      if(size<10) PxxSetFont("symbol8");
      else if(size<12) PxxSetFont("symbol10");
      else if(size<16) PxxSetFont("symbol12");
      else if(size<22) PxxSetFont("symbol16");
      else PxxSetFont("symbol22");
      break;
    case POSTSCRIPT:
      PpsSetFont("Symbol",size);
    }	
  } else {
    switch(PxpView_output) {
    case X11WIN:
      PxxSetFont(font_name);
      break;
    case POSTSCRIPT:
      PpsSetFont(font_name,size);
    }
  }

  return;
}

void PxpDrawString(xx,yy,draw_str)
double xx,yy;
char draw_str[];
/*
draw characters
1997/10/12 T.Handa ver.1.00
*/
{
  switch(PxpView_output) {
  case X11WIN:
    PxxDrawString(xx,yy,draw_str);
    break;
  case POSTSCRIPT:
    PpsDrawString(xx,yy,draw_str);
  }

  return;
}

void PxpDrawJustifiedString(xx,yy,draw_str,x_style,y_style)
double xx,yy;
char draw_str[];
char x_style[],y_style[];
/*
draw strings with justifying their position

remarks:In X11 the width of a character is a half of its height.
  Actual height of the font is 71% of its assigned point size. 
1997/10/23 T.Handa ver.1.00
*/
{
  double shift_x,shift_y;
  int	l_str;

  switch(PxpView_output) {
  case X11WIN:
    l_str=strlen(draw_str);
    if(strcmp(x_style,"right")==0) {
      shift_x=PxxGetPlotSpanX(l_str*PxpView_fontsize)*0.5;
      xx=xx-shift_x;
    } else if(strcmp(x_style,"center")==0) {
      shift_x=PxxGetPlotSpanX(l_str*PxpView_fontsize)*0.25;
      xx=xx-shift_x;
    }
    if(strcmp(y_style,"top")==0) {
      shift_y=PxxGetPlotSpanY(PxpView_fontsize);
      yy=yy-shift_y*0.71;
    } else if(strcmp(y_style,"center")==0) {
      shift_y=PxxGetPlotSpanY(PxpView_fontsize)*0.5;
      yy=yy-shift_y*0.71;
    } else if(strcmp(y_style,"hanging")==0 ||strcmp(y_style,"hang")==0) {
      shift_y=PxxGetPlotSpanY(PxpView_fontsize);
      yy=yy-shift_y;
    }

    PxxDrawString(xx,yy,draw_str);
    break;
  case POSTSCRIPT:
    PpsDrawJustifiedString(xx,yy,draw_str,x_style,y_style);
  }

  return;
}

void PxpDrawJustifiedText(xx,yy,draw_str,x_style,y_style)
double xx,yy;
char draw_str[];
char x_style[],y_style[];
/*
draw strings including CR code with justifying their position
1997/10/22 T.Handa ver.1.00
1999/10/ 2 F.Yamamoto ver.1.12 (debug) CR shift direction
*/
{
  char	*buffer;
  int	l_str;
  int	ii,jj;
  double	dy;

  l_str=strlen(draw_str);
  buffer=(char *)malloc(sizeof(char)*(l_str+1));
  switch(PxpView_output) {
  case X11WIN:
    dy=PxxGetPlotSpanY(PxpView_fontsize);
    break;
  case POSTSCRIPT:
    dy=PxpView_fontsize/CM2INCH/DVI_SCALE;
  }

  jj=0;
  for(ii=0;ii<l_str;ii++) {
    if(draw_str[ii]=='\n') {
      buffer[jj]=NULL;
      PxpDrawJustifiedString(xx,yy,buffer,x_style,y_style);
      jj=0;
      yy=yy-dy;
    } else {
      buffer[jj]=draw_str[ii];
      jj++;
    }
  }
  if(jj!=0) 
    PxpDrawJustifiedString(xx,yy,buffer,x_style,y_style);
  free(buffer);

  return;
}

void PxpFlush()
/*
flush buffer for X11 and actually plotting on screen
(no operation in PS)
1997/10/12 T.Handa ver.1.00
*/
{
  switch(PxpView_output) {
  case X11WIN:
    PxxFlush();
    break;
  }	

  return;
}

/* execute X11 and/or PostScript output */
void PxpDisplayAll(argc,argv, output, filename,	app_class,width,height,plt_fnct,exit_fnct,help_fnct)
/*
create a default type X11 window and/or a PostScript file
1997/10/14 T.Handa ver.1.00
*/
int argc;
char *argv[];
int output;
char *filename;	/* stdout for "-" or "NULL string"; use a temp file if filename==NULL */
char app_class[];	/* name of application class (used in .Xresource) */
double width,height;
void (*plt_fnct)(),(*exit_fnct)(),(*help_fnct)();
{
  void (*plot_function)();

  plot_function=plt_fnct;
  PxpView_output=output;

  switch(output) {
  case X11WIN:
    PxxDisplayAll(argc,argv, app_class,width,height,plt_fnct,exit_fnct,help_fnct);
    break;
  case POSTSCRIPT:
    PpsInitialize(filename,width,height);
    plot_function();	/* user defined plot function */
    PpsPrintPage();
    PpsDoPrint();
  }

  return;
}

void PxpPoint2Range(pt_min,pt_max,range_x,range_y)
PxpPoint pt_min,pt_max;
PxpRange *range_x,*range_y;
/*
convert PxpPoint to PxpRange
1997/12/10 T.Handa
*/
{
  (*range_x).min=pt_min.x;
  (*range_x).max=pt_max.x;
  (*range_y).min=pt_min.y;
  (*range_y).max=pt_max.y;

  return;
}

void PxpRange2Point(range_x,range_y,pt_min,pt_max)
PxpRange range_x,range_y;
PxpPoint *pt_min,*pt_max;
/*
convert PxpRange to PxpPoint
1997/12/10 T.Handa
*/
{
  (*pt_min).x=range_x.min;
  (*pt_min).y=range_y.min;
  (*pt_max).x=range_x.max;
  (*pt_max).y=range_y.max;

  return;
}
