/*
Pxx -- Plot for X11 Window

Library for X11 display
1997/10/26 T.Handa ver.1.00
1998/ 3/ 3 T.Handa ver.1.04 (debug in PxxDisplayAll)
1998/ 5/ 6 T.Handa ver.1.05 (debug in PxxDisplayAll)
2000/ 2/11 T.Handa ver.1.21 (add) PxxWaitMillisec(),PxxFromPreviousVisit()
2000/ 2/24 T.Handa ver.1.60 (add) PxxPixmapInitialize(),PxxDrawCellOnXImage(),PxxLevel2RGBcolorStd(),PxxRGBcolor2pixel(),PxxLevel2pixel()
2000/ 2/28 T.Handa ver.1.61 (debug) PxxExecDraw() ** I must rewrite PxxInitialize? PxpInitialize? and all applications? XFreeGC?
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <math.h>

/* for gettimeofday */
#include <sys/time.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include <Xm/DrawingA.h>
#include <Xm/MainW.h>
#include <Xm/RowColumn.h>
#include <Xm/CascadeB.h>
#include <Xm/PushBG.h>

#include <Pxxlib.h>
#include <Pxplib.h>
#include <Ppslib.h>


static Display	*PxxView_display;
static Window	PxxView_window;
static GC	PxxView_gc;
static int	color_flag=0;
static unsigned int	PxxView_window_width=500,PxxView_window_height=300;
static double	PxxView_width,PxxView_height;

static int	PxxView_Lwidth=1,PxxView_Lstyle=LineSolid,
		PxxView_Lcap=CapNotLast,PxxView_Ljoin=JoinMiter;
static int	max_step4flush=256,step4flush=0;

PxxPoint PxxView_scale={1.0,1.0},PxxView_origin={0.0,0.0};

static void	(*do_plot)(), (*do_exit)(), (*do_help)();

static char	PxxView_printer[256];
static char	PxxView_psfile[256];

/*----for pixmap-----*/
static PxxRGBcolor_long PxxView_RGBshift;
static XColor PxxView_RGBwidth,PxxView_RGBmax;
static int PxxView_Xserver_model;
static unsigned int PxxView_ColorDepth;
static void (*level2color)();


void PxxGetPixelPosition(xx,yy,ii,jj)
double xx,yy;
int *ii,*jj;
{
  *ii=(int)((xx+PxxView_origin.x)*PxxView_scale.x*CM2INCH*DVI_SCALE+0.5);
  *jj=PxxView_window_height-(int)((yy+PxxView_origin.y)*PxxView_scale.y*CM2INCH*DVI_SCALE+0.5);

  return;
}
void PxxGetPlotPosition(ii,jj,xx,yy)
int ii,jj;
double *xx,*yy;
{
  *xx=(double)ii/CM2INCH/DVI_SCALE/PxxView_scale.x-PxxView_origin.x;
  *yy=(double)(PxxView_window_height-jj)/CM2INCH/DVI_SCALE/PxxView_scale.y-PxxView_origin.y;

  return;
}
int PxxGetPixelSpanX(xx)
double xx;
{
  return((int)(xx*CM2INCH*DVI_SCALE*PxxView_scale.x+0.5));
}
int PxxGetPixelSpanY(yy)
double yy;
{
  return((int)(yy*CM2INCH*DVI_SCALE*PxxView_scale.y+0.5));
}
double PxxGetPlotSpanX(ii)
int ii;
{
  return((double)ii/CM2INCH/DVI_SCALE/PxxView_scale.x);
}
double PxxGetPlotSpanY(jj)
int jj;
{
  return((double)jj/CM2INCH/DVI_SCALE/PxxView_scale.y);
}

/* administration functions: get global parameters */
void PxxSetPrinterName(prt_name)
char prt_name[];
/*
output printer name
1997/10/13 T.Handa ver.1.00
*/
{
  strcpy(PxxView_printer,prt_name);
  return;
}
void PxxSetPSFName(psfile_name)
char psfile_name[];
/*
output PostScript file name
1997/10/13 T.Handa ver.1.00
*/
{
  strcpy(PxxView_psfile,psfile_name);
  return;
}

Display *PxxGetDisplay()
/*
get current Display used in X11
1997/10/12 T.Handa ver.1.00
*/
{
  return(PxxView_display);
}
Window	PxxGetWindow()
/*
get current Window used in X11
1997/10/12 T.Handa ver.1.00
*/
{
  return(PxxView_window);
}
GC	PxxGetGC()
/*
get current GraphicContext used in X11
1997/10/12 T.Handa ver.1.00
*/
{
  return(PxxView_gc);
}
void PxxGetWindowSize(x,y)
double *x,*y;
/*
get window size in centi-meter
1997/10/21 T.Handa ver.1.00
*/
{
  double dx,dy;

  *x=PxxGetPlotSpanX(PxxView_window_width);
  *y=PxxGetPlotSpanY(PxxView_window_height);

  return;
}


void PxxGetWindowSizeInPixel(x,y)
int *x,*y;
/*
get window size in pixel
1997/10/21 T.Handa ver.1.00
*/
{
  *x=PxxView_window_width;
  *y=PxxView_window_height;

  return;
}

void PxxGetScale(x,y)
double *x,*y;
/*
get view scale
1997/10/21 T.Handa ver.1.00
*/
{
  *x=PxxView_scale.x;
  *y=PxxView_scale.y;

  return;
}
void PxxGetOrigin(x,y)
double *x,*y;
/*
get view origin
1997/10/20 T.Handa ver.1.00
*/
{
  *x=PxxView_origin.x;
  *y=PxxView_origin.y;

  return;
}

void PxxPresentWindowSize(win_x,win_y,win_w,win_h)
int	*win_x,*win_y;
int	*win_w,*win_h;
/*
get geometry of the current window in X11
1997/12/9 T.Handa ver.1.00
*/
{
  int	err_code;
  int	window_x,window_y;
  Window	root;
  unsigned int border_width;
  unsigned int window_depth;
  double patt_list[1];
  unsigned int	window_width,window_height;

  XGetGeometry(PxxView_display,PxxView_window,&root,&window_x,&window_y,&window_width,&window_height,&border_width,&window_depth);

  *win_x=window_x;
  *win_y=window_y;

  *win_w=window_width;
  *win_h=window_height;

  PxxView_width=window_width;
  PxxView_height=window_height;

  return;
}

/* administration functions: put global parameters */
Display *PxxPutDisplay(disp)
Display *disp;
/*
set Display used in X11
1997/10/12 T.Handa ver.1.00
*/
{
  Display *prev;

  prev=PxxView_display;
  PxxView_display=disp;
  return(prev);
}
Window	PxxPutWindow(win)
Window	win;
/*
set Window used in X11
1997/10/12 T.Handa ver.1.00
*/
{
  Window prev;

  prev=PxxView_window;
  PxxView_window=win;
  return(prev);
}
GC	PxxPutGC(gc)
GC	gc;
/*
set GraphicContext used in X11
1997/10/12 T.Handa ver.1.00
*/
{
  GC prev;

  prev=PxxView_gc;
  PxxView_gc=gc;
  return(prev);
}
void PxxPutWindowSizeInPixel(x,y)
int x,y;
/*
set window size in pixel
1997/12/02 T.Handa ver.1.00
*/
{
  PxxView_window_width=x;
  PxxView_window_height=y;
  return;
}

void	PxxSetPlotFunction(plt_fnt)
void (*plt_fnt)();
/*
assign function name which is executed at click the print-button
1997/10/12 T.Handa ver.1.00
*/
{
  do_plot=plt_fnt;
  return;
}
void	PxxSetExitFunction(exit_fnt)
void (*exit_fnt)();
/*
assign function name which is executed at click the exit-button
1997/10/12 T.Handa ver.1.00
*/
{
  do_exit=exit_fnt;
  return;
}
void	PxxSetHelpFunction(help_fnt)
void (*help_fnt)();
/*
assign function name which is executed at click the help-button
1997/10/12 T.Handa ver.1.00
*/
{
  do_help=help_fnt;
  return;
}

void	PxxSetMaxStepsForFlush(max_step)
int max_step;
/*
set max step number to flush buffer of X11
1997/10/13 T.Handa ver.1.00
*/
{
  max_step4flush=max_step;
  return;
}

void	PxxPutScale(x,y)
double x,y;
/*
set view scale
1997/10/21 T.Handa ver.1.00
*/
{
  PxxView_scale.x=x;
  PxxView_scale.y=y;
  return;
}
void	PxxPutOrigin(x,y)
double x,y;
/*
set view origin
1997/10/21 T.Handa ver.1.00
*/
{
  PxxView_origin.x=x;
  PxxView_origin.y=y;
  return;
}

/* draw graphics */

#define PxxAutoFlush()	\
	{step4flush++;if(step4flush>max_step4flush){PxxFlush();step4flush=0;}}

unsigned long PxxGetColorCode(color)
/*
get the color code number by name
1992/9/7 T.Handa
*/
char *color;
{
  Colormap cmap;
  XColor c0,c1;

  cmap=DefaultColormap(PxxView_display,0);
  XAllocNamedColor(PxxView_display,cmap,color,&c1,&c0);
  return(c1.pixel);
}

void PxxSetColor(color)
char color[];
/*
set line color
1997/10/12 T.Handa ver.1.00
*/
{
  XSetForeground(PxxView_display,PxxView_gc,PxxGetColorCode(color));
  return;
}

void PxxSetGray(density)
double	density;
/*
set line gray
1997/10/12 T.Handa ver.1.00
*/
{
  char buffer[10];
  int dens;

  if(density<0.0) PxxSetColor("black");
  else if(density>1.0) PxxSetColor("white");
  else {
    dens=100-(int)(density*100.0);
    if(dens<=0)
      sprintf(buffer,"gray0",dens);
    else if(dens<10)
      sprintf(buffer,"gray%1d",dens);
    else
      sprintf(buffer,"gray%2d",dens);
    PxxSetColor(buffer);
  }

  return;
}

void PxxSetLineWidth(linewidth)
double linewidth;
/*
set line width
1997/10/15 T.Handa ver.1.00
*/
{
  PxxView_Lwidth=(unsigned int)(linewidth+0.5);
  if(PxxView_Lwidth<1) PxxView_Lwidth=1;
  XSetLineAttributes(PxxView_display,PxxView_gc,PxxView_Lwidth,PxxView_Lstyle,PxxView_Lcap,PxxView_Ljoin);
  return;
}
void PxxSetLinePattern(patt_list,nn,phase)
double patt_list[];
int nn;
double phase;
/*
set line patern
1997/10/14 T.Handa ver.1.00
*/
{
  char dash_list[N_PATT_LIST];
  int ii;
  int i_phase;

  if(nn<2) {	/* solid line */
    PxxView_Lstyle=LineSolid;
    XSetLineAttributes(PxxView_display,PxxView_gc,PxxView_Lwidth,PxxView_Lstyle,PxxView_Lcap,PxxView_Ljoin);
    return;
  }

  if(nn>N_PATT_LIST) nn=N_PATT_LIST;

  for(ii=0;ii<nn;ii++) {
    dash_list[ii]=(char)PxxGetPixelSpanX(patt_list[ii]);
  }
  i_phase=PxxGetPixelSpanY(phase);

  XSetDashes(PxxView_display,PxxView_gc,i_phase,dash_list,nn);

  PxxView_Lstyle=LineOnOffDash;
  XSetLineAttributes(PxxView_display,PxxView_gc,PxxView_Lwidth,PxxView_Lstyle,PxxView_Lcap,PxxView_Ljoin);
  return;
}

void PxxDrawLine(x1,y1,x2,y2)
double x1,y1,x2,y2;
/*
draw a straight line
1997/10/12 T.Handa ver.1.00
*/
{
  int ii1,ii2,jj1,jj2;

  PxxAutoFlush();

  PxxGetPixelPosition(x1,y1,&ii1,&jj1);
  PxxGetPixelPosition(x2,y2,&ii2,&jj2);

  XDrawLine(PxxView_display,PxxView_window,PxxView_gc,ii1,jj1,ii2,jj2);
  return;
}

void PxxDrawLines(pos_list,n_pt)
PxxPoint pos_list[];
int	n_pt;
/*
draw line segments
1997/10/23 T.Handa ver.1.00
*/
{
  XPoint	*point_list;
  int	ii;
  int	ixx,iyy;

  point_list=(XPoint *)malloc(sizeof(XPoint)*n_pt);

  for(ii=0;ii<n_pt;ii++) {
    PxxGetPixelPosition(pos_list[ii].x,pos_list[ii].y,&ixx,&iyy);
    point_list[ii].x=ixx;
    point_list[ii].y=iyy;
  }

  XDrawLines(PxxView_display,PxxView_window,PxxView_gc,point_list,n_pt,CoordModeOrigin);
  free(point_list);
  return;
}

void PxxDrawRectangle(x1,y1,x2,y2,fill)
double x1,y1,x2,y2;
int	fill;
/*
draw a rectangle
1997/10/12 T.Handa ver.1.00
*/
{
  int ii1,ii2,jj1,jj2;
  unsigned int width,height;
  int ii,jj;

  PxxAutoFlush();

  PxxGetPixelPosition(x1,y1,&ii1,&jj1);
  PxxGetPixelPosition(x2,y2,&ii2,&jj2);

  if(ii1>ii2) { ii=ii1; ii1=ii2; ii2=ii; }
  if(jj1>jj2) { jj=jj1; jj1=jj2; jj2=jj; }

  width=ii2-ii1;
  height=jj2-jj1;

  if(fill==NULL)
    XDrawRectangle(PxxView_display,PxxView_window,PxxView_gc,ii1,jj1,width,height);
  else
    XFillRectangle(PxxView_display,PxxView_window,PxxView_gc,ii1,jj1,width,height);
  return;
}

void PxxDrawPolygon(pos_list,n_pt,fill)
PxxPoint pos_list[];
int	n_pt;
int	fill;
/*
draw a polygon
1997/10/23 T.Handa ver.1.00
*/
{
  XPoint	*point_list;
  int	ii;
  int	ixx,iyy;

  point_list=(XPoint *)malloc(sizeof(XPoint)*(n_pt+1));

  for(ii=0;ii<n_pt;ii++) {
    PxxGetPixelPosition(pos_list[ii].x,pos_list[ii].y,&ixx,&iyy);
    point_list[ii].x=ixx;
    point_list[ii].y=iyy;
  }

  if(fill==NULL) {
    if(point_list[0].x!=point_list[n_pt-1].x || point_list[0].y!=point_list[n_pt-1].y) {
      point_list[n_pt].x=point_list[0].x;
      point_list[n_pt].y=point_list[0].y;
      n_pt++;
    }
    XDrawLines(PxxView_display,PxxView_window,PxxView_gc,point_list,n_pt,CoordModeOrigin);
  } else {
    XFillPolygon(PxxView_display,PxxView_window,PxxView_gc,point_list,n_pt,Complex,CoordModeOrigin);
  }
  free(point_list);
  return;
}

void PxxDrawArc(xx,yy,radius,start,theta,fill)
double xx,yy,radius;
double start,theta;
int fill;
/*
draw an arc
1997/10/14 T.Handa ver.1.00
*/
{
  unsigned int width,height;
  int ii,jj;
  int angle1,angle2;

  PxxAutoFlush();

  PxxGetPixelPosition(xx-radius,yy+radius,&ii,&jj);
  width=(unsigned int)PxxGetPixelSpanX(radius*2.0);
  height=(unsigned int)PxxGetPixelSpanY(radius*2.0);

  angle1=(int)(start*64.0+0.5);
  angle2=(int)(theta*64.0+0.5);

  if(fill==NULL)
    XDrawArc(PxxView_display,PxxView_window,PxxView_gc,ii,jj,width,height,angle1,angle2);
  else {
    XSetArcMode(PxxView_display,PxxView_gc,ArcChord);
    XFillArc(PxxView_display,PxxView_window,PxxView_gc,ii,jj,width,height,angle1,angle2);
  }
  return;
}

void PxxDrawSector(xx,yy,radius,start,theta,fill)
double xx,yy,radius;
double start,theta;
int	fill;
/*
draw a sector
1997/10/15 T.Handa ver.1.00
*/
{
  unsigned int width,height;
  int ii,jj;
  int ii1,jj1,ii2,jj2,ii0,jj0;
  int angle1,angle2;

  PxxAutoFlush();

  PxxGetPixelPosition(xx,yy,&ii0,&jj0);
  PxxGetPixelPosition(xx-radius,yy+radius,&ii,&jj);
  width=PxxGetPixelSpanX(radius*2.0);
  height=PxxGetPixelSpanY(radius*2.0);

  angle1=(int)(start*64.0+0.5);
  angle2=(int)(theta*64.0+0.5);

  if(fill==NULL) {
    PxxGetPixelPosition(xx+radius*cos(start/180.0*3.141592),yy+radius*sin(start/180.0*3.141592),&ii1,&jj1);
    PxxGetPixelPosition(xx+radius*cos((start+theta)/180.0*3.141592),yy+radius*sin((start+theta)/180.0*3.141592),&ii2,&jj2);
    XDrawLine(PxxView_display,PxxView_window,PxxView_gc,ii0,jj0,ii1,jj1);
    XDrawArc(PxxView_display,PxxView_window,PxxView_gc,ii,jj,width,height,angle1,angle2);
    XDrawLine(PxxView_display,PxxView_window,PxxView_gc,ii2,jj2,ii0,jj0);
  } else {
    XSetArcMode(PxxView_display,PxxView_gc,ArcPieSlice);
    XFillArc(PxxView_display,PxxView_window,PxxView_gc,ii,jj,width,height,angle1,angle2);
  }
  return;
}

/* plot a character */

void PxxSetFont(font_name)
char font_name[];
/*
set font and its size
1997/10/12 T.Handa ver.1.00
*/
{
  Font font;

  PxxAutoFlush();

  font=XLoadFont(PxxView_display,font_name);
  XSetFont(PxxView_display,PxxView_gc,font);
  return;
}

void PxxDrawString(x1,y1,draw_str)
double x1,y1;
char draw_str[];
/*
draw characters
1997/10/12 T.Handa ver.1.00
*/
{
  int ii,jj;

  PxxAutoFlush();

  PxxGetPixelPosition(x1,y1,&ii,&jj);

  XDrawString(PxxView_display,PxxView_window,PxxView_gc,ii,jj,draw_str,strlen(draw_str));
  return;
}

void PxxFlush()
/*
flush buffer for X11 and plot actually
1997/10/12 T.Handa ver.1.00
*/
{
  XFlush(PxxView_display);
  step4flush=0;

  return;
}

/* define printing */
void PxxExecPrintOut()
/*
print out with PostScript printer
1997/10/21 T.Handa ver.1.00
*/
{
  if(PxxView_psfile[0]==NULL) 
    PpsInitialize(NULL,PxxView_width,PxxView_height);	/* temp file */
  else
    PpsInitialize(PxxView_psfile,PxxView_width,PxxView_height);
  PpsSetPrinterName(PxxView_printer);
  PxpPutOutputMode(POSTSCRIPT);
  (*do_plot)();	/* execute user-defined function */
  PxpPutOutputMode(X11WIN);
  PpsPrintPage();
  PpsDoPrint();

  return;
}

/* define actions for buttons on the default menu bar */
void PxxPushButtonActions(menubarWDG,client_data,call_data)
Widget	menubarWDG;
caddr_t	client_data,call_data;
{
  if(strcmp(client_data,"Exit")==0) {
    if(do_exit!=NULL) do_exit();
    exit(0);
  }
  else if(strcmp(client_data,"Print")==0)
    PxxExecPrintOut();
  else if(strcmp(client_data,"Help")==0)
    if(do_help!=NULL) do_help();

  return;
}

long PxxWaitMillisec(wait_msec)
/*
Wait action in millisecond
2000/2/11 T.Handa
*/
long wait_msec;
{
  struct timeval tv1,tv2;
  struct timezone tz;

  long delta_msec;

  gettimeofday(&tv1,&tz);

  while(1) {
    gettimeofday(&tv2,&tz);

    delta_msec=(tv2.tv_usec-tv1.tv_usec)/1000L;

    if(delta_msec<=0)
      delta_msec=delta_msec+(tv2.tv_sec-tv1.tv_sec)*1000L;

    if(delta_msec>=wait_msec)
      return(delta_msec);
  }
}

long PxxFromPreviousVisit()
/*
get elaps time from the previous visit in millisecond
return 0 in the first call
2000/2/11 T.Handa
*/
{
  static struct timeval prev_tv;
  static int first_call=1;
  struct timeval tv;
  struct timezone tz;

  long delta_msec;

  if(first_call==1) {
    first_call=0;
    gettimeofday(&prev_tv,&tz);
    return 0;
  }

  gettimeofday(&tv,&tz);

  delta_msec=(tv.tv_usec-prev_tv.tv_usec)/1000L;

  if(delta_msec<=0)
    delta_msec=delta_msec+(tv.tv_sec-prev_tv.tv_sec)*1000L;

  return delta_msec;
}

/* execute plotting */
void	PxxExecDraw(windowWDG,graphic_context,callback_data)
Widget	windowWDG;
GC	graphic_context;
XmDrawingAreaCallbackStruct	*callback_data;
{
  XEvent *p_event=callback_data->event;

  int	err_code;
  Window	root;
  int	window_x,window_y;
  unsigned int border_width;
  unsigned int window_depth;
  double patt_list[1];

  /*  if(p_event->xexpose.count > 0) return;*/

  PxxView_display=XtDisplay(windowWDG);
  PxxView_window=XtWindow(windowWDG);

  switch(callback_data->reason) {
    case XmCR_RESIZE :
    case XmCR_EXPOSE :

      XClearWindow(PxxView_display,PxxView_window);
      PxxView_gc=XCreateGC(PxxView_display,PxxView_window,0,0);

      XGetGeometry(PxxView_display,PxxView_window,&root,&window_x,&window_y,&PxxView_window_width,&PxxView_window_height,&border_width,&window_depth);

      /* default set initial values (valid when replotting) */
      PxxSetLineWidth(1.0);
      patt_list[0]=1.0;	/* width 1 point = 1 dot */
      PxxSetLinePattern(patt_list,1,0.0); /* solid line */
      PxxSetFont("7x14");	/* default font */
      PxxSetColor("black");
      PxxPutScale(1.0,1.0);
      PxxPutOrigin(0.0,0.0);

      (*do_plot)();	/* execute user-defined function */

      PxxFlush();

  }
  return;
}

#define MAX_ARG 100
#define	CHAR_SIZE_DEFAULT_RESOURCE	128

/* draw on X11 */
void PxxDisplayAll(argc,argv, 
	app_class,width,height,plt_fnct,exit_fnct,help_fnct)
/*
create a simple window in X11 (as a template of your own application)
1997/10/12 T.Handa ver.1.00
1998/ 3/ 3 T.Handa ver.1.01 (debug)
1998/ 5/ 6 T.Handa ver.1.02 (debug)
*/
int argc;
char *argv[];
char app_class[];	/* name of application class (used in .Xresource) */
double width,height;
void (*plt_fnct)(),(*exit_fnct)(),(*help_fnct)();
{
  Widget  toplevelWDG, main_windowWDG, drawing_areaWDG, manu_barWDG, mbBtn[4];
  Arg     args[10];
  int     ii;
  GC      graphic_context;

  int	width_in_dot,height_in_dot;
  int	mainwin_x,mainwin_y;

  XtAppContext    app_context;
  Arg		fixed_res_args[MAX_ARG];     /* resource arguments */
  Cardinal        fixed_res_n_args;
  static char	window_w[CHAR_SIZE_DEFAULT_RESOURCE],
		window_h[CHAR_SIZE_DEFAULT_RESOURCE];
  static String   fb_res[] = {

    "*mainWindow.width:        500 ",
    "*mainWindow.height:       300 ",
    "*drawArea.background:	white",
    "*menubar.topAttachment:	ATTACH_FORM",
    "*menubar.rightAttachment:	ATTACH_FORM",
    "*menubar.leftAttachment:	ATTACH_FORM",
    NULL};

/* Change initial window size */
  mainwin_x=(int)(width*CM2INCH*DVI_SCALE+0.5);
  mainwin_y=(int)(height*CM2INCH*DVI_SCALE+0.5);
  if(mainwin_x>0.0 && mainwin_x<9999999.0) {
    sprintf(window_w,"*mainWindow.width: %8d",mainwin_x);
    fb_res[0]=window_w;
  }
  if(mainwin_y>0.0 && mainwin_y<9999999.0) {
    sprintf(window_h,"*mainWindow.height:%8d",mainwin_y);
    fb_res[1]=window_h;
  }

  PxxView_width=width;
  PxxView_height=height;
  PxxView_printer[0]=NULL;	/* set default printer */
  PxxView_psfile[0]=NULL;	/* set temporaly file */

/* printer assign by the option parameter -P */
  for(ii=1;ii<argc;ii++) {
    if(strcmp(argv[ii],"-P")==0) PxxSetPrinterName(argv[ii+1]);
    else if(argv[ii][0]=='-' && argv[ii][1]=='P')
      PxxSetPrinterName(&(argv[ii][2]));
  }

/* PostScript file assign by the option parameter -F */
  for(ii=1;ii<argc;ii++) {
    if(strcmp(argv[ii],"-F")==0) PxxSetPSFName(argv[ii+1]);
    else if(argv[ii][0]=='-' && argv[ii][1]=='F')
      PxxSetPSFName(&(argv[ii][2]));
  }

/* entry actual plot function, exit function, help responce function */
  PxxSetPlotFunction(plt_fnct);
  PxxSetExitFunction(exit_fnct);
  PxxSetHelpFunction(help_fnct);

/* create X11 window */
  fixed_res_n_args=0;
  toplevelWDG=XtAppInitialize(&app_context,app_class, NULL, 0, &argc, argv,fb_res,fixed_res_args,fixed_res_n_args);

  main_windowWDG=XmCreateMainWindow(toplevelWDG,"mainWindow",NULL,NULL);
  XtManageChild(main_windowWDG);

/* create menu-bar */
  manu_barWDG=XmCreateMenuBar(main_windowWDG,"menubar",NULL,NULL);
  XtManageChild(manu_barWDG);

  mbBtn[0]=XmCreateCascadeButton(manu_barWDG,"help", args,0);
  XtAddCallback(mbBtn[0],XmNactivateCallback,(XtCallbackProc)PxxPushButtonActions,"Help");
  mbBtn[1]=XmCreateCascadeButton(manu_barWDG,"exit", args,0);
  XtAddCallback(mbBtn[1],XmNactivateCallback,(XtCallbackProc)PxxPushButtonActions, "Exit");
  mbBtn[2]=XmCreateCascadeButton(manu_barWDG,"print", args,0);
  XtAddCallback(mbBtn[2],XmNactivateCallback,(XtCallbackProc)PxxPushButtonActions, "Print");
  XtManageChildren(mbBtn,3);

  XtSetArg(args[0],XmNmenuHelpWidget,(XtArgVal)mbBtn[0]);
  XtSetValues(manu_barWDG,args,1);

/* create drawing area */
  XtSetArg(args[0],XmNresizePolicy,XmRESIZE_ANY);
  drawing_areaWDG=XmCreateDrawingArea(main_windowWDG,"drawArea",args,1);
  XtManageChild(drawing_areaWDG);

/* assign alignment of parts in window */
  XmMainWindowSetAreas(main_windowWDG,manu_barWDG,NULL,NULL,NULL,drawing_areaWDG);

/* realize window */
  XtRealizeWidget(toplevelWDG);

  graphic_context=XCreateGC(XtDisplay(drawing_areaWDG),XtWindow(drawing_areaWDG),0,0);

  XtAddCallback(drawing_areaWDG,XmNexposeCallback,(XtCallbackProc)PxxExecDraw,&drawing_areaWDG);
  XtAddCallback(drawing_areaWDG,XmNresizeCallback,(XtCallbackProc)PxxExecDraw,&drawing_areaWDG);

/* keep the window */
  XtAppMainLoop(app_context);
}

/*-------color Pixmap-------*/
static unsigned short bit_mask_width(long_mask,bit_pos)
unsigned long long_mask; /* <in> */
PxxRange_long *bit_pos; /* <out> */
/*
estimate bit_mask_width
2000/ 2/20 T.Handa
 */
{
  unsigned short bit_width;
  int ii;

  (*bit_pos).min=0;
  (*bit_pos).max=31;

  for(ii=0;ii<32;ii++) {
    if((long_mask & (0x00000001<<ii))!=0) {
      (*bit_pos).min=ii;
      break;
    }
  }

  for(ii=(*bit_pos).min;ii<32;ii++) {
    if((long_mask & (0x00000001<<ii))==0) {
      (*bit_pos).max=ii-1;
      break;
    }
  }

  bit_width=(*bit_pos).max-(*bit_pos).min+1;

  return bit_width;
}

XImage *PxxPixmapInitialize(image_plane,depth,window_w,window_h)
char *image_plane;
unsigned int depth;
unsigned int window_w,window_h;
/*
Initialize paramters for pixmap on X11
return) XImage address
2000/ 2/23 T.Handa
*/
{
  XImage *x_image;
  int ii;
  PxxRange_long bit_pos[3];
  unsigned short bit_width[3];
  int ix,iy,iz;

  Visual *visual;
  int byte4pixel;

  byte4pixel=(depth-1)/8+1; /* byte for a pixel, that is byte deps */

/* create XImage for pixmap */
  visual=DefaultVisual(PxxView_display,0);
  depth=DefaultDepth(PxxView_display,0);

  x_image=XCreateImage(PxxView_display,visual,depth,ZPixmap,0,image_plane,(unsigned int)window_w,(unsigned int)window_h,byte4pixel*8,window_w*byte4pixel);

  PxxView_ColorDepth=(*x_image).depth;

  PxxView_RGBwidth.red=bit_mask_width((*x_image).red_mask,&(bit_pos[0]));
  PxxView_RGBwidth.green=bit_mask_width((*x_image).green_mask,&(bit_pos[1]));
  PxxView_RGBwidth.blue=bit_mask_width((*x_image).blue_mask,&(bit_pos[2]));

  PxxView_RGBmax.red=0x00000001<<PxxView_RGBwidth.red;
  PxxView_RGBmax.green=0x00000001<<PxxView_RGBwidth.green;
  PxxView_RGBmax.blue=0x00000001<<PxxView_RGBwidth.blue;

  PxxView_RGBshift.red=0x00000001<<bit_pos[0].min;
  PxxView_RGBshift.green=0x00000001<<bit_pos[1].min;
  PxxView_RGBshift.blue=0x00000001<<bit_pos[2].min;

  window_w=(*x_image).width;
  window_h=(*x_image).height;
  byte4pixel=((*x_image).depth-1)/8+1;

  if(PxxView_RGBmax.red*PxxView_RGBmax.green*PxxView_RGBmax.blue<=1L) {
    PxxView_Xserver_model=PxxINDEX_COLOR_MODEL;
/* clear image plane by white */

    for(iy=0;iy<window_h;iy++) {
      for(ix=0;ix<window_w;ix++) {
	for(iz=0;iz<byte4pixel;iz++) {
	  (*x_image).data[iz+(ix+iy*window_w)*byte4pixel]=0x00;
	}
      }
    }
  } else {
    PxxView_Xserver_model=PxxDIRECT_COLOR_MODEL;

/* clear image plane by white */
    for(iy=0;iy<window_h;iy++) {
      for(ix=0;ix<window_w;ix++) {
	for(iz=0;iz<byte4pixel;iz++) {
	  (*x_image).data[iz+(ix+iy*window_w)*byte4pixel]=0xff;
	}
      }
    }
  }

  return x_image;
}

void PxxDrawCellOnXImage(ximage,x0,y0,x1,y1,pixel)
XImage *ximage; /* <out> */
double x0,y0,x1,y1; /* <in> 4 corners */
unsigned long pixel; /* <in> */
/*
fill a rectangle region of XImage with pixel.
out range is automatically corrected.
2000/ 2/24 T.Handa
 */
{
  double xx,yy;
  int ix[2],iy[2];
  int ii,jj;

  PxxGetPixelPosition(x0,y0,&(ix[0]),&(iy[0]));
  PxxGetPixelPosition(x1,y1,&(ix[1]),&(iy[1]));

/* sort */
  if(ix[0]>ix[1]) {
    ii=ix[0]; ix[0]=ix[1]; ix[1]=ii;
  }
  if(iy[0]>iy[1]) {
    ii=iy[0]; iy[0]=iy[1]; iy[1]=ii;
  }

/* totally out range */
  if(ix[0]<0 && ix[1]<0) return;
  if(ix[0]>=ximage->width && ix[1]>=ximage->width) return;
  if(iy[0]<0 && iy[1]<0) return;
  if(iy[0]>=ximage->height && iy[1]>=ximage->height) return;

/* out range correction */
  if(ix[0]<0) ix[0]=0;
  if(ix[1]<0) ix[1]=0;
  if(ix[0]>=ximage->width) ix[0]=ximage->width - 1;
  if(ix[1]>=ximage->width) ix[1]=ximage->width - 1;

  if(iy[0]<0) iy[0]=0;
  if(iy[1]<0) iy[1]=0;
  if(iy[0]>=ximage->height) iy[0]=ximage->height - 1;
  if(iy[1]>=ximage->height) iy[1]=ximage->height - 1;

/* actually fill */
  for(jj=iy[0];jj<=iy[1];jj++) {
    for(ii=ix[0];ii<=ix[1];ii++) {
      XPutPixel(ximage,ii,jj,pixel);
    }
  }

  return;
}

void PxxLevel2RGBcolorStd(rgb_color,color_level)
PxxRGBcolor *rgb_color; /* <out> 0.0<=RGB<=1.0 */
double color_level; /* <in> 0.0<=color_level<=1.0 */
/*
standard conversion function from color_level to PxxRGBcolor
2000/ 2/22 T.Handa
*/
{
  (*rgb_color).red=0.0;
  (*rgb_color).green=color_level;
  (*rgb_color).blue=0.0;

  return;
}

unsigned long PxxRGBcolor2pixel(x_color)
PxxRGBcolor x_color; /* <in> RGB color, 0.0<=RGB<1.0 */
/*
convert PxxRGBcolor to unsigned long pixel value for XImage
2000/2/21 T.Handa
*/
{
  unsigned long pixel,red,green,blue;

  red=(unsigned long)(x_color.red*(double)(PxxView_RGBmax.red-1));
  green=(unsigned long)(x_color.green*(double)(PxxView_RGBmax.green-1));
  blue=(unsigned long)(x_color.blue*(double)(PxxView_RGBmax.blue-1));

  pixel=red*PxxView_RGBshift.red | green*PxxView_RGBshift.green | blue*PxxView_RGBshift.blue;

  return pixel;
}

unsigned long PxxLevel2pixel(lvl2clr,color_level)
void (*lvl2clr)(); /* <in> function */
double color_level; /* <in> 0.0<=color_level<=1.0 */
/*
convert a color level to pixel value through a function "lvl2clr()" 
2000/ 2/24 T.Handa
 */
{
  unsigned long pixel;
  PxxRGBcolor rgb_color;

  if(PxxView_Xserver_model==PxxINDEX_COLOR_MODEL) {
    pixel=(unsigned long)(color_level*pow(2.0,(double)PxxView_ColorDepth));
  } else {
    PxxLevel2RGBcolorStd(&rgb_color,color_level);
    pixel=PxxRGBcolor2pixel(rgb_color);
  }

  return pixel;
}
